# name: systemd
# desc: function for systemd byte size
# auth: Kai Burghardt <wiz at KaiBurghardt.de>
#  vim: set tw=140 ts=4 noet bs=eol,start,indent fdm=marker fdl=0:

# {include:file:yrd/bytes.yrd}
function systemd::bytes (
	Struct[{
			bytes                         => Optional[Integer],
			kibibytes                     => Optional[Integer],
			mebibytes                     => Optional[Integer],
			gibibytes                     => Optional[Integer],
			tebibytes                     => Optional[Integer],
		}] $amount = {
			bytes                         => 0,
			kibibytes                     => 0,
			mebibytes                     => 0,
			tebibytes                     => 0,
		},
	Boolean                  $restrict_input_values         = true,
	Boolean                  $restrict_non_negative_result  = true,
) {
	# I N P U T   M A P P I N G
	$_bytes                        = $amount['bytes'] ? {
		undef                         => 0,
		default                       => $amount['bytes'],
	}
	
	$_kibibytes                    = $amount['kibibytes'] ? {
		undef                         => 0,
		default                       => $amount['kibibytes'],
	}
	
	$_mebibytes                    = $amount['mebibytes'] ? {
		undef                         => 0,
		default                       => $amount['mebibytes'],
	}
	
	$_gibibytes                    = $amount['gibibytes'] ? {
		undef                         => 0,
		default                       => $amount['gibibytes'],
	}
	
	$_tebibytes                    = $amount['tebibytes'] ? {
		undef                         => 0,
		default                       => $amount['tebibytes'],
	}
	
	# I N P U T   V A L I D A T I O N
	if $restrict_input_values {
		if ($_bytes < 0) or ($_bytes > 1023) {
			fail('bytes must be within [0, 1023]')
		}
		if ($_kibibytes < 0) or ($_kibibytes > 1023) {
			fail('kibibytes must be within [0, 1023]')
		}
		if ($_mebibytes < 0) or ($_mebibytes > 1023) {
			fail('mebibytes must be within [0, 1023]')
		}
		if ($_gibibytes < 0) or ($_gibibytes > 1023) {
			fail('gibibytes must be within [0, 1023]')
		}
		# tebibytes is this function's largest accepted prefix
		# do not restrict its upper bound
		if ($_tebibytes < 0) {
			fail('tebibytes must be within [0, infty)')
		}
	}
	
	# C A L C U L A T I O N
	$_result = $_bytes +
		1024 * $_kibibytes +
		1024 * 1024 * $_mebibytes +
		1024 * 1024 * 1024 * $_gibibytes +
		1024 * 1024 * 1024 * 1024 * $_tebibytes
	
	# O U T P U T   V A L I D A T I O N
	if ($restrict_non_negative_result) and ($result < 0) {
		fail('non-negative restriction on: got negative result')
	}
	
	# R E T U R N
	$_result
}
# EOF
