# name: systemd
# desc: init manifest for systemd module
# auth: Kai Burghardt <wiz at KaiBurghardt.de>
#  vim: set tw=140 ts=4 noet bs=eol,start,indent fdm=marker fdl=0:

# {include:file:doc/init.yrd}
class systemd (
	# G E N E R A L #########################################################
	# packages
	Optional[String[1]]      $systemd_package_name          = $systemd::params::systemd_package_name,
	String[1]                $systemd_package_version       = $systemd::params::systemd_package_version,
	
	# puppet resource management
	Boolean                  $systemd_local_gens_dir        = false,
	Boolean                  $collect_systemd_resources     = true,
	Boolean                  $remove_rather_purge_conflicts = false,
	
	# S E R V I C E S #######################################################
	# systemd-networkd(8)
	Optional[Enum['running', 'stopped']]
	                         $systemd_networkd_ensure       = undef,
	Optional[Boolean]        $systemd_networkd_enable       = undef,
	Boolean                  $manage_networkd_competitors   = false,
	Array[String[1]]         $networkd_competitors_packages = $systemd::params::networkd_competitors_packages,
	# systemd-resolved(8)
	Optional[Enum['running', 'stopped']]
	                         $systemd_resolved_ensure       = undef,
	Optional[Boolean]        $systemd_resolved_enable       = undef,
	Boolean                  $manage_resolv_conf_symlink    = false,
	# systemd-timesyncd(8)
	Optional[Enum['running', 'stopped']]
	                         $systemd_timesyncd_ensure      = undef,
	Optional[Boolean]        $systemd_timesyncd_enable      = undef,
	Boolean                  $manage_timesyncd_conflicts    = false,
	Array[String[1]]         $timesyncd_package_conflicts   = $systemd::params::timesyncd_package_conflicts,
	
	# C O N F I G U R A T I O N   F I L E S #################################
	# configuration files introduced by systemd 
	Boolean                  $manage_loc_systemd_conf       = false,
	Boolean                  $purge_loc_systemd_conf_dir    = false,
	Boolean                  $manage_loc_journald_conf      = false,
	Boolean                  $purge_loc_journald_conf_dir   = false,
	Boolean                  $manage_loc_logind_conf        = false,
	Boolean                  $purge_loc_logind_conf_dir     = false,
	Boolean                  $purge_loc_networkd_conf_dir   = false,
	Boolean                  $manage_loc_resolved_conf      = false,
	Boolean                  $purge_loc_resolved_conf_dir   = false,
	Boolean                  $manage_loc_timesyncd_conf     = false,
	Boolean                  $purge_loc_timesyncd_conf_dir  = false,
	Boolean                  $manage_loc_system_preset_dir  = false,
	Boolean                  $purge_loc_system_preset_dir   = false,
	Boolean                  $manage_loc_bootchart_conf     = false,
	Boolean                  $purge_loc_bootchart_conf_dir  = false,
	Boolean                  $manage_loc_sleep_conf         = false,
	Boolean                  $purge_loc_sleep_conf_dir      = false,
	# more configuration files
	Boolean                  $manage_modules_load_dir       = false,
	Boolean                  $manage_machine_info           = false,
	Enum['hostnamectl', 'template']
	                         $machine_info_provider         = 'hostnamectl',
	Boolean                  $manage_sysusers_dir           = false,
	# annexed [in a disdaining connotation like 'claiming territory'] conf files
	Boolean                  $etc_network_interfaces_note   = false,
	String[1]                $etc_network_interfaces_erb    = $systemd::params::etc_network_interfaces_erb,
	Boolean                  $manage_sysctl_conf            = false,
	Boolean                  $manage_sysctl_directory       = false,
	Boolean                  $manage_udev_conf              = false,
	Boolean                  $manage_locale_conf            = false,
	Enum['localectl', 'template']
	                         $locale_conf_provider          = 'template',
	Boolean                  $manage_localtime              = false,
	Enum['timedatectl', 'file']
	                         $localtime_provider            = 'timedatectl',
	
	# puppet resource management
	String[1]                $systemd_system_conf_erb       = $systemd::params::systemd_system_conf_erb,
	String[1]                $systemd_journald_conf_erb     = $systemd::params::systemd_journald_conf_erb,
	String[1]                $systemd_logind_conf_erb       = $systemd::params::systemd_logind_conf_erb,
	String[1]                $systemd_resolved_conf_erb     = $systemd::params::systemd_resolved_conf_erb,
	String[1]                $systemd_timesyncd_conf_erb    = $systemd::params::systemd_timesyncd_conf_erb,
	String[1]                $systemd_bootchart_conf_erb    = $systemd::params::systemd_bootchart_conf_erb,
	String[1]                $systemd_sleep_conf_erb        = $systemd::params::systemd_sleep_conf_erb,
	Optional[String[1]]      $sysctl_conf_template          = undef,
	String[1]                $udev_conf_template            = $systemd::params::udev_conf_template,
	
	# C O N F I G U R A T I O N   P A R A M E T E R S #######################
	# system.conf
	Optional[Variant[
			Integer[0, 7],
			Enum['emerg', 'alert', 'crit', 'err', 'warning', 'notice', 'info', 'debug']
		]] $manager_log_level = undef,
	Optional[Enum['console', 'journal', 'kmsg', 'journal-or-kmsg', 'null']]
	                         $manager_log_target            = undef,
	Optional[Boolean]        $manager_log_color             = undef,
	Optional[Boolean]        $manager_log_location          = undef,
	Optional[Boolean]        $manager_dump_core             = undef,
	Optional[Boolean]        $manager_crash_shell           = undef,
	Optional[Boolean]        $manager_show_status           = undef,
	Optional[Integer]        $manager_crash_ch_vt           = undef,
	Optional[Enum[
			'inherit',
			'null',
			'tty',
			'journal',
			'journal+console',
			'syslog',
			'syslog+console',
			'kmsg',
			'kmsg+console'
		]] $manager_default_stdout = undef,
	Optional[Enum[
			'inherit',
			'null',
			'tty',
			'journal',
			'journal+console',
			'syslog',
			'syslog+console',
			'kmsg',
			'kmsg+console'
		]] $manager_default_stderr = undef,
	Optional[Array[Integer[0], 1]]
	                         $manager_cpu_affinity          = undef,
	Optional[Array[String[1], 0]]
	                         $manager_join_controllers      = undef,
	Optional[Integer[0]]     $manager_runtime_watchdog_sec  = undef,
	Optional[Integer[0]]     $manager_shutdown_watchdog_sec = undef,
	Optional[Struct[{
			inverted_selection            => Boolean,
	    	capabilities                  => Array[Pattern[
					/^CAP_AUDIT_(CONTROL|READ|WRITE)$/,
					/^CAP_BLOCK_SUSPEND$/,
					/^CAP_CHOWN$/,
					/^CAP_DAC_(OVERRIDE|READ_SEARCH)$/,
					/^CAP_FOWNER$/,
					/^CAP_FSETID$/,
					/^CAP_IPC_(LOCK|OWNER)$/,
					/^CAP_KILL$/,
					/^CAP_LEASE$/,
					/^CAP_LINUX_IMMUTABLE$/,
					/^CAP_MAC_(ADMIN|OVERRIDE)$/,
					/^CAP_MKNOD$/,
					/^CAP_NET_(ADMIN|BIND_SERVICE|BROADCAST|RAW)$/,
					/^CAP_SETGID$/,
					/^CAP_SETFCAP$/,
					/^CAP_SETUID$/,
					/^CAP_SYS_(ADMIN|BOOT|CHROOT|MODULE|NICE|PACCT|PTRACE|RAWIO|RESOURCE|TIME|TTY_CONFIG)$/,
					/^\|?!?CAP_SYSLOG$/,
					/^\|?!?CAP_WAKE_ALARM$/
				], 1],
		}]] $manager_capability_bound_set  = undef,
	Optional[Array[Enum['x86', 'x86-64', 'x32', 'arm', 'native'], 1]]
	                         $manager_syscall_architectures = undef,
	Optional[Integer[0]]     $manager_timer_slack_nanosec   = undef,
	Optional[Integer[0]]     $manager_default_timer_accurac = undef,
	Optional[Integer[0]]     $manager_default_to_start_sec  = undef,
	Optional[Integer[0]]     $manager_default_to_stop_sec   = undef,
	Optional[Integer[0]]     $manager_default_restart_sec   = undef,
	Optional[Integer[0]]     $manager_default_start_lim_int = undef,
	Optional[Integer[0]]     $manager_default_start_lim_brs = undef,
	Optional[Hash[
			String[1],
			Variant[String, Integer, Array[Variant[String[1], Integer], 1]],
			1
		]] $manager_default_environment = undef,
	Optional[Boolean]        $manager_default_cpu_acct      = undef,
	Optional[Boolean]        $manager_default_block_io_acct = undef,
	Optional[Boolean]        $manager_default_memory_acct   = undef,
	Optional[Variant[Enum['infinity'], Integer[0]]]
	                         $manager_default_lim_cpu       = undef,
	Optional[Variant[Enum['infinity'], Integer[0]]]
	                         $manager_default_lim_fsize     = undef,
	Optional[Variant[Enum['infinity'], Integer[0]]]
	                         $manager_default_lim_data      = undef,
	Optional[Variant[Enum['infinity'], Integer[0]]]
	                         $manager_default_lim_stack     = undef,
	Optional[Variant[Enum['infinity'], Integer[0]]]
	                         $manager_default_lim_core      = undef,
	Optional[Variant[Enum['infinity'], Integer[0]]]
	                         $manager_default_lim_rss       = undef,
	Optional[Variant[Enum['infinity'], Integer[0]]]
	                         $manager_default_lim_nofile    = undef,
	Optional[Variant[Enum['infinity'], Integer[0]]]
	                         $manager_default_lim_as        = undef,
	Optional[Variant[Enum['infinity'], Integer[0]]]
	                         $manager_default_lim_nproc     = undef,
	Optional[Variant[Enum['infinity'], Integer[0]]]
	                         $manager_default_lim_memlock   = undef,
	Optional[Variant[Enum['infinity'], Integer[0]]]
	                         $manager_default_lim_locks     = undef,
	Optional[Variant[Enum['infinity'], Integer[0]]]
	                         $manager_default_lim_sigpendin = undef,
	Optional[Variant[Enum['infinity'], Integer[0]]]
	                         $manager_default_lim_msgqueue  = undef,
	Optional[Variant[Enum['infinity'], Integer[0]]]
	                         $manager_default_lim_nice      = undef,
	Optional[Variant[Enum['infinity'], Integer[0]]]
	                         $manager_default_lim_rtprio    = undef,
	Optional[Variant[Enum['infinity'], Integer[0]]]
	                         $manager_default_lim_rttime    = undef,
	# journald.conf
	Optional[Enum['volatile', 'persistent', 'auto', 'none']]
	                         $journal_storage               = undef,
	Optional[Boolean]        $journal_compress              = undef,
	Optional[Boolean]        $journal_seal                  = undef,
	Optional[Enum['uid', 'login', 'none']]
	                         $journal_split_mode            = undef,
	Optional[Integer[0]]     $journal_rate_limit_interval   = undef,
	Optional[Integer[0]]     $journal_rate_limit_burst      = undef,
	Optional[Integer[0]]     $journal_system_max_use        = undef,
	Optional[Integer[0]]     $journal_system_keep_free      = undef,
	Optional[Integer[0]]     $journal_system_max_file_size  = undef,
	Optional[Integer[0]]     $journal_runtime_max_use       = undef,
	Optional[Integer[0]]     $journal_runtime_keep_free     = undef,
	Optional[Integer[0]]     $journal_runtime_max_file_size = undef,
	Optional[Integer[0]]     $journal_max_file_sec          = undef,
	Optional[Integer[0]]     $journal_max_retention_sec     = undef,
	Optional[Integer[0]]     $journal_sync_interval_sec     = undef,
	Optional[Boolean]        $journal_forward_to_syslog     = undef,
	Optional[Boolean]        $journal_forward_to_kmsg       = undef,
	Optional[Boolean]        $journal_forward_to_console    = undef,
	Optional[Boolean]        $journal_forward_to_wall       = undef,
	Optional[Variant[
			Integer[0, 7],
			Enum['emerg', 'alert', 'crit', 'err', 'warning', 'notice', 'info', 'debug']
		]] $journal_max_level_store = undef,
	Optional[Variant[
			Integer[0, 7],
			Enum['emerg', 'alert', 'crit', 'err', 'warning', 'notice', 'info', 'debug']
		]] $journal_max_level_syslog = undef,
	Optional[Variant[
			Integer[0, 7],
			Enum['emerg', 'alert', 'crit', 'err', 'warning', 'notice', 'info', 'debug']
		]] $journal_max_level_kmsg = undef,
	Optional[Variant[
			Integer[0, 7],
			Enum['emerg', 'alert', 'crit', 'err', 'warning', 'notice', 'info', 'debug']
		]] $journal_max_level_console = undef,
	Optional[Variant[
			Integer[0, 7],
			Enum['emerg', 'alert', 'crit', 'err', 'warning', 'notice', 'info', 'debug']
		]] $journal_max_level_wall = undef,
	Optional[Pattern[/^\/\S+$/]]
	                         $journal_tty_path              = undef,
	# logind.conf
	Optional[Integer[0]]     $login_n_auto_vts              = undef,
	Optional[Integer[0]]     $login_reserve_vt              = undef,
	Optional[Boolean]        $login_kill_user_processes     = undef,
	Optional[Variant[Array[Pattern[/^[^[:blank:]\n]+$/], 1], Array[Enum[''], 1, 1]]]
	                         $login_kill_only_users         = undef,
	Optional[Variant[Array[Pattern[/^[^[:blank:]\n]+$/], 1], Array[Enum[''], 1, 1]]]
	                         $login_kill_exclude_users      = undef,
	Optional[Enum['ignore',
			'poweroff',
			'reboot',
			'halt',
			'kexec',
			'suspend',
			'hibernate',
			'hybrid-sleep',
			'lock'
		]] $login_idle_action = undef,
	Optional[Integer[0]]     $login_idle_action_sec         = undef,
	Optional[Integer[0]]     $login_inhibit_delay_max_sec   = undef,
	Optional[Enum[
			'ignore',
			'poweroff',
			'reboot',
			'halt',
			'kexec',
			'suspend',
			'hibernate',
			'hybrid-sleep',
			'lock'
		]] $login_handle_power_key = undef,
	Optional[Enum[
			'ignore',
			'poweroff',
			'reboot',
			'halt',
			'kexec',
			'suspend',
			'hibernate',
			'hybrid-sleep',
			'lock'
		]] $login_handle_suspend_key = undef,
	Optional[Enum[
			'ignore',
			'poweroff',
			'reboot',
			'halt',
			'kexec',
			'suspend',
			'hibernate',
			'hybrid-sleep',
			'lock'
		]] $login_handle_hibernate_key = undef,
	Optional[Enum[
			'ignore',
			'poweroff',
			'reboot',
			'halt',
			'kexec',
			'suspend',
			'hibernate',
			'hybrid-sleep',
			'lock'
		]] $login_handle_lid_sw = undef,
	Optional[Enum[
			'ignore',
			'poweroff',
			'reboot',
			'halt',
			'kexec',
			'suspend',
			'hibernate',
			'hybrid-sleep',
			'lock'
		]] $login_handle_lid_sw_docked = undef,
	Optional[Boolean]        $login_power_key_ign_inhib     = undef,
	Optional[Boolean]        $login_suspend_key_ign_inhib   = undef,
	Optional[Boolean]        $login_hibernate_key_ign_inhib = undef,
	Optional[Boolean]        $login_lid_switch_ign_inhib    = undef,
	Optional[Integer[0]]     $login_holdoff_timeout_sec     = undef,
	Optional[Integer[0]]     $login_runtime_directory_size  = undef,
	Optional[Boolean]        $login_remove_ipc              = undef,
	# resolved.conf
	Optional[Array[Variant[
			Array[Integer[0, 255], 4, 4],
			Array[Integer[0x0000, 0xFFFF], 8, 8]
		], 0]] $resolve_dns = undef,
	Optional[Array[Variant[
			Array[Integer[0, 255], 4, 4],
			Array[Integer[0x0000, 0xFFFF], 8, 8]
		], 0]] $resolve_fallback_dns = undef,
	Optional[Array[Pattern[/^[^[:blank:]\n]+$/], 1]]
	                         $resolve_domains               = undef,
	Optional[Variant[Boolean, Enum['resolve']]]
	                         $resolve_llmnr                 = undef,
	Optional[Variant[Boolean, Enum['allow-downgrade']]]
	                         $resolve_dnssec                = undef,
	Optional[Boolean]        $resolve_cache                 = undef,
	# timesyncd.conf
	Optional[Array[Variant[
			String[1],
			Array[Integer[0, 255], 4, 4],
			Array[Integer[0x0000, 0xFFFF], 8, 8]
		], 0]] $time_ntp = undef,
	Optional[Array[Variant[
			String[1],
			Array[Integer[0, 255], 4, 4],
			Array[Integer[0x0000, 0xFFFF], 8, 8]
		], 0]] $time_fallback_ntp = undef,
	# sleep.conf
	Optional[Array[String]]  $sleep_suspend_mode            = undef,
	Optional[Array[String]]  $sleep_hibernate_mode          = undef,
	Optional[Array[String]]  $sleep_hybrid_sleep_mode       = undef,
	Optional[Array[String]]  $sleep_suspend_state           = undef,
	Optional[Array[String]]  $sleep_hibernate_state         = undef,
	Optional[Array[String]]  $sleep_hybrid_sleep_state      = undef,
	# bootchart.conf
	Optional[Integer[0]]     $bootchart_samples             = undef,
	Optional[Variant[Integer[1], Float[0.1]]]
	                         $bootchart_frequency           = undef,
	Optional[Boolean]        $bootchart_relative            = undef,
	Optional[Boolean]        $bootchart_filter              = undef,
	Optional[Pattern[/^\/\S+\/$/]]
	                         $bootchart_output              = undef,
	Optional[Pattern[/^\/\S+$/]]
	                         $bootchart_init                = undef,
	Optional[Boolean]        $bootchart_plot_memory_usage   = undef,
	Optional[Boolean]        $bootchart_plot_entropy_graph  = undef,
	Optional[Integer[1]]     $bootchart_scale_x             = undef,
	Optional[Integer[1]]     $bootchart_scale_y             = undef,
	Optional[Boolean]        $bootchart_control_group       = undef,
	Optional[Boolean]        $bootchart_per_cpu             = undef,
	
	# machine-info
	Optional[Variant[Default, Pattern[/^\S+$/]]]
	                         $machine_info_pretty_hostname  = undef,
	Optional[Variant[Default, Pattern[/^\S+$/]]]
	                         $machine_info_icon_name        = undef,
	Optional[Variant[
			Enum[
				'desktop',
				'laptop',
				'tablet',
				'handset',
				'watch',
				'embedded',
				'vm',
				'container',
				'server'
			],
			Default
		]] $machine_info_chassis = undef,
	Optional[Variant[Default, Pattern[/^\w+$/]]]
	                         $machine_info_deployment       = undef,
	Optional[Variant[Default, Pattern[/^[^\n]+$/, /^'.*'$/]]]
	                         $machine_info_location         = undef,
	# locale.conf
	Optional[Pattern[/^[\w_.-]+$/]]
	                         $locale_conf_lang              = undef,
	Optional[Pattern[/^[\w_.-]+$/]]
	                         $locale_conf_language          = undef,
	Optional[Pattern[/^[\w_.-]+$/]]
	                         $locale_conf_lc_ctype          = undef,
	Optional[Pattern[/^[\w_.-]+$/]]
	                         $locale_conf_lc_numeric        = undef,
	Optional[Pattern[/^[\w_.-]+$/]]
	                         $locale_conf_lc_time           = undef,
	Optional[Pattern[/^[\w_.-]+$/]]
	                         $locale_conf_lc_collate        = undef,
	Optional[Pattern[/^[\w_.-]+$/]]
	                         $locale_conf_lc_monetary       = undef,
	Optional[Pattern[/^[\w_.-]+$/]]
	                         $locale_conf_lc_messages       = undef,
	Optional[Pattern[/^[\w_.-]+$/]]
	                         $locale_conf_lc_paper          = undef,
	Optional[Pattern[/^[\w_.-]+$/]]
	                         $locale_conf_lc_name           = undef,
	Optional[Pattern[/^[\w_.-]+$/]]
	                         $locale_conf_lc_address        = undef,
	Optional[Pattern[/^[\w_.-]+$/]]
	                         $locale_conf_lc_telephone      = undef,
	Optional[Pattern[/^[\w_.-]+$/]]
	                         $locale_conf_lc_measurement    = undef,
	Optional[Pattern[/^[\w_.-]+$/]]
	                         $locale_conf_lc_identification = undef,
	# localtime
	Optional[Pattern[/^[\w\/+-_]+[\w]$/]]
	                         $localtime                     = 'UTC',
	# udev.conf
	Optional[Variant[Enum['err', 'info', 'debug'], Integer[0, 7]]]
	                         $udev_conf_udev_log            = undef,
) inherits systemd::params {
	Class['systemd::install'] -> Class['systemd::config'] -> Class['systemd::daemon']
	contain 'systemd::install'
	contain 'systemd::config'
	contain 'systemd::daemon'
	
	# addresses were implemented as array of integer
	# for trivial range check
	# [ought to be strings (since you don't calculate with them)]
}
# EOF
