# name: install systemd
# desc: install manifest for systemd module
# auth: Kai Burghardt <wiz at KaiBurghardt.de>
#  vim: set tw=140 ts=4 noet bs=eol,start,indent fdm=marker:

# user documentation as YARD documentation {{{
# 
# This is a private class.
# 
# * It manages several `package` resources.
# * Nothing else.
# 
# }}}
class systemd::install {
	# V A R I A B L E   I N I T
	
	$_default_removal_method       = $systemd::remove_rather_purge_conflicts ? {
		true                          => 'absent',
		default                       => 'purged',
	}
	
	
	# M A I N   P A C K A G E
	
	# use the systemd package to ensure the directory /etc/systemd/system exist
	if ($systemd::systemd_package_name != undef) {
		package { 'systemd':
			ensure                        => $systemd::systemd_package_version,
			name                          => $systemd::systemd_package_name,
		}
	}
	
	
	# C O N F L I C T I N G   P A C K A G E S
	
	if $systemd::manage_networkd_competitors {
		if ($systemd::networkd_competitors_packages != undef) and
		   ($systemd::networkd_competitors_packages != []) {
			package { $systemd::networkd_competitors_packages:
				ensure                        => $_default_removal_method,
			}
		}
	}
	
	if $systemd::manage_timesyncd_conflicts {
		if ($systemd::timesyncd_package_conflicts != undef) and
		   ($systemd::timesyncd_package_conflicts != []) {
			package { $systemd::timesyncd_package_conflicts:
				ensure                        => $_default_removal_method,
			}
		}
	}
}
# EOF
