# name: systemd
# desc: definition manifest for systemd link type
# auth: Kai Burghardt <wiz at KaiBurghardt.de>
#  vim: set tw=140 ts=4 noet bs=eol,start,indent fdm=marker fdl=0:

# {include:file:yrd/link.yrd}
define systemd::link (
	# P U P P E T   R E S O U R C E   M A N A G E M E N T
	Optional[Variant[Enum['absent', 'present'], Default]]
	                         $ensure                        = default,
	# <prefix_name>.link file (and dependent)
	Variant[Pattern[/^\/\S+$/], Default]
	                         $path                          = default,
	Optional[Variant[String[1], Default]]
	                         $mode                          = default,
	Optional[Variant[Integer[0], String[1], Default]]
	                         $owner                         = default,
	Optional[Variant[Integer[0], String[1], Default]]
	                         $group                         = default,
	Variant[String[1], Default]
	                         $content_template              = default,
	Optional[Variant[String[1], Default]]
	                         $validate_cmd                  = undef,
	Optional[String[1]]
	                         $validate_replacement          = undef,
	Optional[Enum[
			'md5',
			'md5lite',
			'sha256',
			'sha256lite',
			'mtime',
			'ctime',
			'none'
		]] $checksum = undef,
	Optional[Boolean]        $show_diff                     = undef,
	Optional[Variant[Boolean, String]]
	                         $backup                        = undef,
	# L I N K
	# match
	Optional[Array[Integer[0x00, 0xFF], 6, 6]]
	                         $match_mac_address             = undef,
	Optional[Array[Pattern[/^\S+$/]]]
	                         $match_original_name           = undef,
	Optional[Array[Pattern[/^\S+$/]]]
	                         $match_path                    = undef,
	Optional[Array[Pattern[/^\S+$/]]]
	                         $match_driver                  = undef,
	Optional[Array[Pattern[/^\S+$/]]]
	                         $match_type                    = undef,
	Optional[Pattern[/^\S+$/]]
	                         $match_host                    = undef,
	Optional[Variant[
			Boolean,
			Pattern[/^(vm|container)$/],
			Pattern[/^(qemu|kvm|zvm|vmware|microsoft|oracle|xen|bochs|uml)$/],
			Pattern[/^(openvz|lxc|lxc-libvirt|systemd-nspawn|docker)$/]
		]] $match_virtualization = undef,
	Optional[Pattern[/^!?\S+$/]]
	                         $match_kernel_cmdline          = undef,
	Optional[Pattern[
			/^x86(-64)?$/,
			/^ppc(-le)?$/,
			/^ppc64(-le)?$/,
			/^ia64$/,
			/^parisc(64)?$/,
			/^s390x?$/,
			/^sparc(64)?$/,
			/^mips(-le)?$/,
			/^mips64(-le)?$/,
			/^alpha$/,
			/^arm(-be)?$/,
			/^arm64(-be)?$/,
			/^sh(64)?$/,
			/^m86k$/,
			/^tilegx$/,
			/^cris$/
		]] $match_architecture = undef,
	# link
	Optional[String]         $link_description              = undef,
	Optional[String]         $link_alias                    = undef,
	Optional[Enum['persistent', 'random']]
	                         $link_mac_address_policy       = undef,
	Optional[Array[Integer[0x00, 0xFF], 6, 6]]
	                         $link_mac_address              = undef,
	Optional[Array[Enum['kernel', 'database', 'onboard', 'slot', 'path', 'mac']]]
	                         $link_name_policy              = undef,
	Optional[String]         $link_name                     = undef,
	Optional[Integer[0]]     $link_mtu_bytes                = undef,
	Optional[Integer[0]]     $link_bits_per_second          = undef,
	Optional[Enum['half', 'full']]
	                         $link_duplex                   = undef,
	Optional[Enum['phy', 'magic', 'off']]
	                         $link_wake_on_lan              = undef,
) {
	if ($link_mac_address != undef) and ($link_mac_address_policy != undef) {
		warning('link_mac_address only used, if no link_mac_address_policy')
	}
}
# EOF
