# name: systemd
# desc: definition manifest for systemd netdev type
# auth: Kai Burghardt <wiz at KaiBurghardt.de>
#  vim: set tw=140 ts=4 noet bs=eol,start,indent fdm=marker fdl=0:

# {include:file:yrd/netdev.yrd}
define systemd::netdev (
	# P U P P E T   R E S O U R C E   M A N A G E M E N T
	Optional[Variant[Enum['absent', 'present'], Default]]
	                         $ensure                        = default,
	# <prefix_name>.link file (and dependent)
	Variant[Pattern[/^\/\S+$/], Default]
	                         $path                          = default,
	Optional[Variant[String[1], Default]]
	                         $mode                          = default,
	Optional[Variant[Integer[0], String[1], Default]]
	                         $owner                         = default,
	Optional[Variant[Integer[0], String[1], Default]]
	                         $group                         = default,
	Variant[String[1], Default]
	                         $content_template              = default,
	Optional[Variant[String[1], Default]]
	                         $validate_cmd                  = undef,
	Optional[String[1]]
	                         $validate_replacement          = undef,
	Optional[Enum[
			'md5',
			'md5lite',
			'sha256',
			'sha256lite',
			'mtime',
			'ctime',
			'none'
		]] $checksum = undef,
	Optional[Boolean]        $show_diff                     = undef,
	Optional[Variant[Boolean, String]]
	                         $backup                        = undef,
	# N E T D E V ###########################################################
	# M A T C H
	Optional[Pattern[/^[^[:blank:]\n]+$/]]
	                         $match_host                    = undef,
	Optional[Variant[
			Boolean,
			Pattern[/^\|?!?(vm|container)$/],
			Pattern[/^\|?!?(qemu|kvm|zvm|vmware|microsoft|oracle|xen|bochs|uml)$/],
			Pattern[/^\|?!?(openvz|lxc|lxc-libvirt|systemd-nspawn|docker)$/]
		]] $match_virtualization = undef,
	Optional[Pattern[/^!?[^[:blank:]\n]+$/]]
	                         $match_kernel_cmdline          = undef,
	Optional[Pattern[
			/^x86(-64)?$/,
			/^ppc(-le)?$/,
			/^ppc64(-le)?$/,
			/^ia64$/,
			/^parisc(64)?$/,
			/^s390x?$/,
			/^sparc(64)?$/,
			/^mips(-le)?$/,
			/^mips64(-le)?$/,
			/^alpha$/,
			/^arm(-be)?$/,
			/^arm64(-be)?$/,
			/^sh(64)?$/,
			/^m86k$/,
			/^tilegx$/,
			/^cris$/
		]] $match_architecture = undef,
	# N E T D E V
	Optional[Pattern[/^[^\n]+$/]]
	                         $netdev_description            = undef,
	Pattern[/^\w+$/]         $netdev_name                   = undef,
	Pattern[
			/^bond$/,
			/^bridge$/,
			/^gre(tap)?$/,
			/^ip6(gre|tnl|gretap)$/,
			/^ip(ip|vlan)$/
			/^mac(vlan|vtap)$/,
			/^sit$/,
			/^tap$/,
			/^tun$/,
			/^veth$/,
			/^vlan$/,
			/^vti6?$/,
			/^vxlan$/,
			/^dummy$/
		] $netdev_kind = undef,
	Optional[Integer[0]]     $netdev_mtu_bytes              = undef,
	Optional[Array[Integer[0x00, 0xFF], 6, 6]]
	                         $netdev_mac_address            = undef,
	# V L A N
	Optional[Integer[0, 4094]]
	                         $vlan_id                       = undef,
	# M A C V L A N
	Optional[Enum['private', 'vepa', 'bridge', 'passthru']]
	                         $macvlan_mode                  = undef,
	# M A C V T A P
	Optional[Enum['private', 'vepa', 'bridge', 'passthru']]
	                         $macvtap_mode                  = undef,
	# I P V L A N
	Optional[Enum['L2', 'L3']]
	                         $ipvlan_mode                   = undef,
	# V X L A N
	Optional[String]         $vxlan_id                      = undef,
	Optional[String]         $vxlan_group                   = undef,
	Optional[Integer[0, 255]]
	                         $vxlan_tos                     = undef,
	Optional[Integer[0, 255]]
	                         $vxlan_ttl                     = undef,
	Optional[Boolean]        $vxlan_mac_learning            = undef,
	Optional[Integer[0]]     $vxlan_fdb_ageing_sec          = undef,
	Optional[Boolean]        $vxlan_arp_proxy               = undef,
	Optional[Boolean]        $vxlan_l2_miss_notification    = undef,
	Optional[Boolean]        $vxlan_l3_miss_notification    = undef,
	Optional[Boolean]        $vxlan_route_short_circuit     = undef,
	Optional[Boolean]        $vxlan_udp_check_sum           = undef,
	# sic! "UDP6ZeroChecksumTx=" & "UDP6ZeroCheckSumRx=" [varying case use]
	Optional[Boolean]        $vxlan_udp6_zero_checksum_tx   = undef,
	Optional[Boolean]        $vxlan_udp6_zero_check_sum_rx  = undef,
	Optional[Boolean]        $vxlan_group_policy_extension  = undef,
	# T U N N E L
	Optional[Variant[
			Struct[{
				ipv6_address_words            => Array[Integer[0x0000, 0xFFFF], 8, 8],
			}],
			Struct[{
				ipv4_address_octets           => Array[Integer[0, 255], 4, 4],
			}]
		]] $tunnel_local = undef,
	Optional[String]         $tunnel_remote                 = undef,
	Optional[Integer[0, 255]]
	                         $tunnel_tos                    = undef,
	Optional[Integer[0, 255]]
	                         $tunnel_ttl                    = undef,
	Optional[Boolean]        $tunnel_discover_path_mtu      = undef,
	Optional[Variant[Integer[0x0, 0xFFFFF], Enum['inherit']]]
	                         $tunnel_ipv6_flow_label        = undef,
	Optional[Boolean]        $tunnel_copy_dscp              = undef,
	Optional[Enum['ip6ip6', 'ipip6', 'any']]
	                         $tunnel_mode                   = undef,
	# P E E R
	Optional[Pattern[/^\w+$/]]
	                         $peer_name                     = undef,
	Optional[Array[Integer[0x00, 0xFF], 6, 6]]
	                         $peer_mac_address              = undef,
	# T U N
	Optional[Boolean]        $tun_one_queue                 = undef,
	Optional[Boolean]        $tun_multi_queue               = undef,
	Optional[Boolean]        $tun_packet_info               = undef,
	Optional[Boolean]        $tun_vnet_header               = undef,
	Optional[Variant[Pattern[/^[^[:blank:]\n]+$/], Integer[0]]]
	                         $tun_user                      = undef,
	Optional[Variant[Pattern[/^[^[:blank:]\n]+$/], Integer[0]]]
	                         $tun_group                     = undef,
	# T A P
	Optional[Boolean]        $tap_one_queue                 = undef,
	Optional[Boolean]        $tap_multi_queue               = undef,
	Optional[Boolean]        $tap_packet_info               = undef,
	Optional[Boolean]        $tap_vnet_header               = undef,
	Optional[Variant[Pattern[/^[^[:blank:]\n]+$/], Integer[0]]]
	                         $tap_user                      = undef,
	Optional[Variant[Pattern[/^[^[:blank:]\n]+$/], Integer[0]]]
	                         $tap_group                     = undef,
	# B O N D
	Optional[Enum[
			'active-backup',
			'balance-xor',
			'broadcast',
			'802.3ad',
			'balance-tlb',
			'balance-alb',
			'balance-rr'
		]] $bond_mode = undef,
	Optional[Enum[
			'layer2',
			'layer3+4',
			'layer2+3',
			'encap2+3',
			'802.3ad',
			'encap3+4'
		]] $bond_transmit_hash_policy = undef,
	Optional[Enum['fast', 'slow']]
	                         $bond_lacp_transmit_rate       = undef,
	Optional[Integer[0]]     $bond_mii_monitor_sec          = undef,
	Optional[Integer[0]]     $bond_up_delay_sec             = undef,
	Optional[Integer[0]]     $bond_down_delay_sec           = undef,
	Optional[Integer[0x1, 0x7FFFFFFF]]
	                         $bond_learn_packet_interval_s  = undef,
	Optional[Enum['stable', 'bandwidth', 'count']]
	                         $bond_ad_select                = undef,
	Optional[Enum['active', 'follow', 'none']]
	                         $bond_fail_over_mac_policy     = undef,
	Optional[Enum['active', 'backup', 'all', 'none']]
	                         $bond_arp_validate             = undef,
	Optional[Integer[0]]     $bond_arp_interval_sec         = undef,
	Optional[Array[
			Struct[{
				ipv4_address_octets           => Array[Integer[0, 255], 4, 4],
			}],
		1, 16]] $bond_arp_ip_targets = undef,
	Optional[Enum['any', 'all']]
	                         $bond_arp_all_targets          = undef,
	Optional[Enum['always', 'better', 'failure']]
	                         $bond_primary_reselect_policy  = undef,
	Optional[Integer[0x00, 0xFF]]
	                         $bond_resend_igmp              = undef,
	Optional[Integer[0x0000, 0xFFFF]]
	                         $bond_packets_per_slave        = undef,
	Optional[Integer[0x00, 0xFF]]
	                         $bond_gratuitous_arp           = undef,
	Optional[Boolean]        $bond_all_slaves_active        = undef,
	Optional[Integer[0]]     $bond_min_links                = undef,
) {
	#                                                                        #
	# I M P L E M E N T A T I O N                                            #
	#                                                                        #
	
	# I N P U T   C H E C K S
	if (($netdev_kind == 'vlan') and ($vlan_id == undef)) {
		fail('if $netdev_kind = \'vlan\', you\'ve gotta specify $vlan_id')
	}
	
	if (($netdev_kind == 'veth') and ($peer_name == undef)) {
		fail('if $netdev_kind is \'veth\', you\'ve gotta specifiy $peer_name')
	}
}
# EOF
