# name: systemd
# desc: definition manifest for systemd network type
# auth: Kai Burghardt <wiz at KaiBurghardt.de>
#  vim: set tw=140 ts=4 noet bs=eol,start,indent fdm=marker fdl=0:

define systemd::network (
	# P U P P E T   R E S O U R C E   M A N A G E M E N T
	Optional[Variant[Enum['absent', 'present'], Default]]
	                         $ensure                        = default,
	# <prefix_name>.network file (and dependent)
	Variant[Pattern[/^\/\S+$/], Default]
	                         $path                          = default,
	Optional[Variant[String[1], Default]]
	                         $mode                          = default,
	Optional[Variant[Integer[0], String[1], Default]]
	                         $owner                         = default,
	Optional[Variant[Integer[0], String[1], Default]]
	                         $group                         = default,
	Variant[String[1], Default]
	                         $content_template              = default,
	Optional[Enum[
			'md5',
			'md5lite',
			'sha256',
			'sha256lite',
			'mtime',
			'ctime',
			'none'
		]] $checksum = undef,
	Optional[Boolean]        $show_diff                     = undef,
	Optional[Variant[Boolean, String]]
	                         $backup                        = undef,
	# N E T W O R K #########################################################
	# M A T C H
	Optional[Array[Integer[0x00, 0xFF], 6, 6]]
	                         $match_mac_address             = undef,
	Optional[Array[Pattern[/^\S+$/]]]
	                         $match_path                    = undef,
	Optional[Array[Pattern[/^\S+$/]]]
	                         $match_driver                  = undef,
	Optional[Array[Pattern[/^\S+$/]]]
	                         $match_type                    = undef,
	Optional[Array[Pattern[/^\S+$/]]]
	                         $match_name                    = undef,
	Optional[Variant[
			Boolean,
			Pattern[/^(vm|container)$/],
			Pattern[/^(qemu|kvm|zvm|vmware|microsoft|oracle|xen|bochs|uml)$/],
			Pattern[/^(openvz|lxc|lxc-libvirt|systemd-nspawn|docker)$/]
		]] $match_virtualization = undef,
	Optional[Pattern[/^!?\S+$/]]
	                         $match_kernel_cmdline          = undef,
	Optional[Pattern[
			/^x86(-64)?$/,
			/^ppc(-le)?$/,
			/^ppc64(-le)?$/,
			/^ia64$/,
			/^parisc(64)?$/,
			/^s390x?$/,
			/^sparc(64)?$/,
			/^mips(-le)?$/,
			/^mips64(-le)?$/,
			/^alpha$/,
			/^arm(-be)?$/,
			/^arm64(-be)?$/,
			/^sh(64)?$/,
			/^m86k$/,
			/^tilegx$/,
			/^cris$/
		]] $match_architecture = undef,
	# L I N K
	Optional[Array[Integer[0x00, 0xFF], 6, 6]]
	                         $link_mac_address              = undef,
	Optional[Integer[0]]     $link_mtu_bytes                = undef,
	# N E T W O R K
	Optional[String]         $network_description           = undef,
	# while for systemd.unit(5) the strings `1`, `yes`, `true` and `on`
	# are equivalent, there is no such statement for systemd.network(5).
	# ergo force the literal string 'yes' or 'no' for following parameters
	Optional[Enum['yes', 'no', 'ipv4', 'ipv6']]
	                         $network_dhcp                  = undef,
	Optional[Boolean]        $network_dhcp_server           = undef,
	Optional[Enum['yes', 'no', 'ipv4', 'ipv6']]
	                         $network_link_local_addressing = undef,
	Optional[Boolean]        $network_ipv4_ll_route         = undef,
	Optional[Array[Integer[0x0000, 0xFFFF], 8, 8]]
	                         $network_ipv6_token            = undef,
	Optional[Variant[Boolean, Enum['resolve']]]
	                         $network_llmnr                 = undef,
	Optional[Boolean]        $network_lldp                  = undef,
	Optional[Array[Integer[0, 65535]]]
	                         $network_bind_carrier          = undef,
	Optional[Array[Variant[
			Struct[{
				ipv4_address_octets           => Array[Integer[0, 255], 4, 4],
				cidr_mask                     => Integer[0, 32],
			}],
			Struct[{
				ipv6_address_words            => Array[Integer[0x0000, 0xFFFF], 8, 8],
				cidr_mask                     => Integer[0, 128],
			}]
		], 1]] $network_address = undef,
	Optional[Array[Variant[
			Array[Integer[0, 255], 4, 4],
			Array[Integer[0x0000, 0xFFFF], 8, 8]
		], 1]] $network_gateway = undef,
	Optional[Array[Variant[
			Array[Integer[0, 255], 4, 4],
			Array[Integer[0x0000, 0xFFFF], 8, 8]
		], 1]] $network_dns = undef,
	Optional[Array[String[1]]]
	                         $network_domains               = undef,
	Optional[Array[Variant[
			String[1],
			Array[Integer[0, 255], 4, 4],
			Array[Integer[0x0000, 0xFFFF], 8, 8]
		]]] $network_ntp = undef,
	Optional[Variant[Boolean, Enum['ipv4', 'ipv6', 'kernel']]]
	                         $network_ip_forward            = undef,
	Optional[Boolean]        $network_ip_masquerade         = undef,
	Optional[Variant[Boolean, Enum['prefer-public', 'kernel']]]
	                         $network_ipv6_privacy_extensns = undef,
	Optional[String[1]]      $network_bridge                = undef,
	Optional[String[1]]      $network_bond                  = undef,
	Optional[Array[String[1], 1]]
	                         $network_vlan                  = undef,
	Optional[Array[String[1], 1]]
	                         $network_macvlan               = undef,
	Optional[Array[String[1], 1]]
	                         $network_vxlan                 = undef,
	Optional[Array[String[1], 1]]
	                         $network_tunnel                = undef,
	# A D D R E S S
	Optional[Array[Struct[{
			address                       => Array[Variant[
					Struct[{
						ipv4_address_octets           => Array[Integer[0, 255], 4, 4],
						cidr_mask                     => Integer[0, 32],
					}],
					Struct[{
						ipv6_address_words            => Array[Integer[0x0000, 0xFFFF], 8, 8],
						cidr_mask                     => Integer[0, 128],
					}]
				], 1],
			peer                          => Optional[Array[Variant[
					Struct[{
						ipv4_address_octets           => Array[Integer[0, 255], 4, 4],
						cidr_mask                     => Integer[0, 32],
					}],
					Struct[{
						ipv6_address_words            => Array[Integer[0x0000, 0xFFFF], 8, 8],
						cidr_mask                     => Integer[0, 128],
					}]
				], 1]],
			broadcast                     => Optional[Array[Variant[
					Array[Integer[0, 255], 4, 4],
					Array[Integer[0x0000, 0xFFFF], 8, 8]
				], 1]],
			label                         => Optional[String[1]],
		}], 1]] $address = undef,
	# R O U T E
	Optional[Array[Struct[{
			gateway                       => Optional[Array[Variant[
					Array[Integer[0, 255], 4, 4],
					Array[Integer[0x0000, 0xFFFF], 8, 8]
				]]],
			destination                   => Optional[Array[Variant[
					Struct[{
						ipv4_address_octets           => Array[Integer[0, 255], 4, 4],
						cidr_mask                     => Optional[Integer[0, 32]],
					}],
					Struct[{
						ipv6_address_words            => Array[Integer[0x0000, 0xFFFF], 8, 8],
						cidr_mask                     => Optional[Integer[0, 128]],
					}]
				], 1]],
			source                        => Optional[Array[Variant[
					Struct[{
						ipv4_address_octets           => Array[Integer[0, 255], 4, 4],
						cidr_mask                     => Optional[Integer[0, 32]],
					}],
					Struct[{
						ipv6_address_words            => Array[Integer[0x0000, 0xFFFF], 8, 8],
						cidr_mask                     => Optional[Integer[0, 128]],
					}]
				], 1]],
			metric                        => Optional[Integer[0]],
			scope                         => Optional[Enum['global', 'link', 'host']],
		}], 1]] $route = undef,
	# D H C P
	Optional[Boolean]        $dhcp_use_dns                  = undef,
	Optional[Boolean]        $dhcp_use_ntp                  = undef,
	Optional[Boolean]        $dhcp_use_mtu                  = undef,
	Optional[Boolean]        $dhcp_send_hostname            = undef,
	Optional[Boolean]        $dhcp_use_hostname             = undef,
	Optional[String]         $dhcp_hostname                 = undef,
	Optional[Boolean]        $dhcp_use_domains              = undef,
	Optional[Boolean]        $dhcp_use_routes               = undef,
	Optional[Boolean]        $dhcp_use_critical_connection  = undef,
	Optional[Enum['mac', 'duid']]
	                         $dhcp_client_identifier        = undef,
	Optional[String[1]]      $dhcp_vendor_class_identifier  = undef,
	Optional[Boolean]        $dhcp_request_broadcast        = undef,
	Optional[Boolean]        $dhcp_route_metric             = undef,
	# B R I D G E
	Optional[Boolean]        $bridge_unicast_flood          = undef,
	Optional[Boolean]        $bridge_hair_pin               = undef,
	Optional[Boolean]        $bridge_use_bpdu               = undef,
	Optional[Boolean]        $bridge_fast_leave             = undef,
	Optional[Boolean]        $bridge_allow_port_to_be_root  = undef,
	Optional[Integer]        $bridge_cost                   = undef,
	# B R I D G E F D B
	Optional[Array[Struct[{
			mac_address                   => Array[Integer[0x00, 0xFF], 6, 6],
			vland_id                      => Optional[String[1]],
		}], 1]] $bridge_fdb = undef,
) {
	#                                                                        #
	# I M P L E M E N T A T I O N                                            #
	#                                                                        #
	
	# load (system-dependent) default values from params.pp
	include systemd::params
	
	# we do not create files with spaces in their name
	if ($title =~ /\s/) and ($path == default) {
		fail('title may not contain spaces unless path specified')
	}
	
	if ($title =~ /\//) and ($path == default) {
		fail('title may not contain slashes unless path specified')
	}
	
	
	# A T T R I B U T E S   F O R   N E T W O R K   F I L E
	$_ensure                       = ($ensure == default) ? {
		true                          => 'file',
		default                       => $ensure ? {
			'present'                     => 'file',
			default                       => $ensure,
		},
	}
	
	$_prefix_name                  = $title
	
	$_dirname                      = $systemd::params::systemd_local_network_dir
	
	$_basename                     = ($path == default) ? {
		true                          => "${_prefix_name}.network",
		default                       => inline_template("<%= File.basename(scope['path']) %>"),
	}
	
	$_path                         = ($path == default) ? {
		true                          => "${_dirname}${_basename}",
		default                       => $path,
	}
	
	$_title                        = "${_prefix_name}_network"
	
	$_mode                         = ($mode == default) ? {
		true                          => $systemd::params::systemd_unit_default_mode,
		default                       => $mode,
	}
	
	$_owner                        = ($owner == default) ? {
		true                          => $systemd::params::systemd_unit_default_owner,
		default                       => $owner,
	}
	
	$_group                        = ($group == default) ? {
		true                          => $systemd::params::systemd_unit_default_group,
		default                       => $group,
	}
	
	$_content_template             = ($content_template == default) ? {
		true                          => $systemd::params::systemd_network_template,
		default                       => $content_template,
	}
	
	
	# R E S O U R C E   D E F A U L T S
	# their values are independent from anything
	File {
		show_diff                     => $show_diff,
		backup                        => $backup,
	}
	
	
	# R E S O U R C E   D E C L A R A T I O N S
	if ($_ensure != undef) {
		file { $_title:
			ensure                        => $_ensure,
			path                          => $_path,
			mode                          => $_mode,
			owner                         => $_owner,
			group                         => $_group,
			content                       => template($_content_template),
		}
	# end of ensure-something-block
	}
}
# EOF
