# name: systemd parameters
# desc: parameter manifest for systemd module
# auth: Kai Burghardt <wiz at KaiBurghardt.de>
#  vim: set tw=140 ts=4 noet bs=eol,start,indent fdm=marker fdl=0:

# user documentation as YARD documentation {{{
# 
# @abstract Traditional definition of default values.
# 
# @note The {systemd::data} function has not yet been enabled by default.
#       If you'd like to try it, you have to modify the `metadata.json`.
# 
# @see systemd::data
# }}}
class systemd::params {
	#                                                                       #
	# G E N E R A L   V A L U E S                                           #
	#                                                                       #
	$systemd_local_units_dir       = '/etc/systemd/system/'
	$systemd_local_network_dir     = '/etc/systemd/network/'
	$systemd_runtime_units_dir     = '/run/systemd/system/'
	$systemd_package_units_dir     = '/lib/systemd/system/'
	$systemd_system_unit_load_path = [
			$systemd_local_units_dir,
			$systemd_runtime_units_dir,
			$systemd_package_units_dir,
		]
	# [Device], [Target] are empty sections. reserve 'em anyway
	# [Snaphot] are not configured via files. reserve it anyway
	$systemd_known_section_names   = [
			'Address',
			'Automount',
			'Bond',
			'Bootchart',
			'Bridge',
			'BridgeFDB',
			'Device',
			'DHCP',
			'Install',
			'IPVLAN',
			'Journal',
			'Link',
			'Login',
			'MACVLAN',
			'MACVTAP',
			'Manager',
			'Match',
			'Mount',
			'Netdev',
			'Network',
			'Path',
			'Peer',
			'Resolve',
			'Route',
			'Scope',
			'Service',
			'Slice',
			'Snapshot',
			'Socket',
			'Swap',
			'Tap',
			'Target',
			'Time',
			'Timer',
			'Tun',
			'Tunnel',
			'Unit',
			'VLAN',
			'VXLAN',
		]
	
	
	#                                                                       #
	# C L A S S   D E F A U L T   P A R A M E T E R S                       #
	#                                                                       #
	$systemd_package_name          = 'systemd'
	$systemd_package_version       = present
	
	$networkd_competitors_packages = [
			'ifupdown',
			'ifupdown2',
			'libnm-glib4', # network-manager
		]
	
	$timesyncd_package_conflicts   = [
			'chrony',
			'ntp',
			'openntpd',
		]
	
	$systemd_local_gens_dir        = false
	$etc_network_interfaces_erb    = 'systemd/etc_network_interfaces.erb'
	$systemd_system_conf_erb       = 'systemd/system_conf.erb'
	$systemd_journald_conf_erb     = 'systemd/journald_conf.erb'
	$systemd_logind_conf_erb       = 'systemd/logind_conf.erb'
	$systemd_resolved_conf_erb     = 'systemd/resolved_conf.erb'
	$systemd_timesyncd_conf_erb    = 'systemd/timesyncd_conf.erb'
	$systemd_bootchart_conf_erb    = 'systemd/bootchart_conf.erb'
	$systemd_sleep_conf_erb        = 'systemd/sleep_conf.erb'
	$udev_conf_template            = 'systemd/udev_conf.erb'
	
	
	#                                                                       #
	# C U S T O M   D E F I N E D   T Y P E   D E F A U L T S               #
	#                                                                       #
	$systemd_unit_default_mode     = '0644'
	#$systemd_unit_default_owner    = $::facts['os']['distro']['id'] ? {
	$systemd_unit_default_owner    = $::osfamily ? {
		'FreeBSD'                     => 'root',
		'Debian'                      => 'root',
		default                       => 0,
	}
	#$systemd_unit_default_group    = $::facts['os']['distro']['id'] ? {
	$systemd_unit_default_group    = $::osfamily ? {
		'FreeBSD'                     => 'wheel',
		'Debian'                      => 'root',
		default                       => 0,
	}
	
	$systemd_unit_template         = 'systemd/unit.erb'
	$systemd_unit_preamble_templte = 'systemd/preamble.erb'
	$systemd_unit_epilog_template  = undef
	$systemd_unit_validate_cmd     = '/usr/bin/systemd-analyze verify %'
	$systemd_unit_wants_def_mode   = '0755'
	$systemd_unit_wants_def_owner  = $systemd_unit_default_owner
	$systemd_unit_wants_def_group  = $systemd_unit_default_group
	$systemd_unit_req_def_mode     = $systemd_unit_wants_def_mode
	$systemd_unit_req_def_owner    = $systemd_unit_default_owner
	$systemd_unit_req_def_group    = $systemd_unit_default_group
	$systemd_unit_conf_def_mode    = $systemd_unit_wants_def_mode
	$systemd_unit_conf_def_owner   = $systemd_unit_default_owner
	$systemd_unit_conf_def_group   = $systemd_unit_default_group
	
	$systemd_service_template      = 'systemd/service.erb'
	$systemd_socket_template       = 'systemd/socket.erb'
	$systemd_mount_template        = 'systemd/mount.erb'
	$systemd_automount_template    = 'systemd/automount.erb'
	$systemd_swap_template         = 'systemd/swap.erb'
	$systemd_target_template       = $systemd_unit_template
	$systemd_path_template         = 'systemd/path.erb'
	$systemd_timer_template        = 'systemd/timer.erb'
	$systemd_slice_template        = 'systemd/slice.erb'
	$systemd_preset_template       = 'systemd/preset.erb'
	$systemd_link_template         = 'systemd/link.erb'
	$systemd_network_template      = 'systemd/network.erb'
	$systemd_netdev_template       = 'systemd/netdev.erb'
	$systemd_tmpfiles_template     = 'systemd/tmpfiles.erb'
	$systemd_modules_load_template = 'systemd/modules_load.erb'
	$systemd_sysctl_conf_template  = 'systemd/sysctl_conf.erb'
	$systemd_sysusers_template     = 'systemd/sysusers.erb'
}
# EOF
