# name: systemd
# desc: definition manifest for systemd service type
# auth: Kai Burghardt <wiz at KaiBurghardt.de>
#  vim: set tw=140 ts=4 noet bs=eol,start,indent fdm=marker fdl=0:

# do not edit systemd/manifests/service.pp. it's generated

define systemd::service (
	# P U P P E T   R E S O U R C E   M A N A G E M E N T
	Optional[Variant[Enum['absent', 'present'], Default]]
	                         $ensure                        = default,
	# <prefix_name>.service file (and dependent)
	Variant[Pattern[/^\/\S*[^[:blank:]\/]$/], Default]
	                         $path                          = default,
	Optional[Variant[String[1], Default]]
	                         $mode                          = default,
	Optional[Variant[Integer[0], String[1], Default]]
	                         $owner                         = default,
	Optional[Variant[Integer[0], String[1], Default]]
	                         $group                         = default,
	Variant[String[1], Default]
	                         $content_template              = default,
	Optional[Variant[String[1], Default]]
	                         $validate_cmd                  = undef,
	Optional[String[1]]
	                         $validate_replacement          = undef,
	Optional[Enum[
			'md5',
			'md5lite',
			'sha256',
			'sha256lite',
			'mtime',
			'ctime',
			'none'
		]] $checksum = undef,
	Optional[Boolean]        $show_diff                     = undef,
	Optional[Variant[Boolean, String]]
	                         $backup                        = undef,
	String                   $blame                         = $systemd::params::systemd_unit_default_owner,
	# <unit_name>.wants/ directory
	Boolean                  $manage_wants_dir              = false,
	Optional[Variant[String[1], Default]]
	                         $wants_dir_mode                = default,
	Optional[Variant[Integer[0], String[1], Default]]
	                         $wants_dir_owner               = default,
	Optional[Variant[Integer[0], String[1], Default]]
	                         $wants_dir_group               = default,
	Boolean                  $wants_dir_complete_control    = false,
	# <unit_name>.requires/ directory
	Boolean                  $manage_requires_dir           = false,
	Optional[Variant[String[1], Default]]
	                         $requires_dir_mode             = default,
	Optional[Variant[Integer[0], String[1], Default]]
	                         $requires_dir_owner            = default,
	Optional[Variant[Integer[0], String[1], Default]]
	                         $requires_dir_group            = default,
	Boolean                  $requires_dir_complete_control = false,
	# <unit_name>.d/ directory
	Boolean                  $manage_conf_dir               = false,
	Optional[Variant[String[1], Default]]
	                         $conf_dir_mode                 = default,
	Optional[Variant[Integer[0], String[1], Default]]
	                         $conf_dir_owner                = default,
	Optional[Variant[Integer[0], String[1], Default]]
	                         $conf_dir_group                = default,
	Boolean                  $conf_dir_complete_control     = false,
	# U N I T
	# unit metadata
	Optional[String]         $unit_description              = undef,
	Optional[Array[Pattern[/^(https?:\/\/|file:|info:|man:)\S+$/, /^$/], 1]]
	                         $unit_documentation            = undef,
	# unit dependencies
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_requires                 = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_requires_overridable     = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_requisite                = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_requisite_overridable    = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_wants                    = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_binds_to                 = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_part_of                  = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_conflicts                = undef,
	# unit ordering
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_before                   = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_after                    = undef,
	# unit behavior
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_on_failure               = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_propagates_reload_to     = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_reload_propagated_from   = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_joins_namespace_of       = undef,
	# absolute path (starts with slash)
	Optional[Array[Pattern[/^\/\S+$/], 1]]
	                         $unit_requires_mount_for       = undef,
	Optional[Enum[
			'fail',
			'replace',
			'replace-irreversibly',
			'isolate',
			'flush',
			'ignore-dependencies',
			'ignore-requirements'
		]] $unit_on_failure_job_mode = undef,
	Optional[Boolean]        $unit_ignore_on_isolate        = undef,
	Optional[Boolean]        $unit_ignore_on_snapshot       = undef,
	Optional[Boolean]        $unit_stop_when_unneeded       = undef,
	Optional[Boolean]        $unit_refuse_manual_start      = undef,
	Optional[Boolean]        $unit_refuse_manual_stop       = undef,
	Optional[Boolean]        $unit_allow_isolate            = undef,
	Optional[Boolean]        $unit_default_dependencies     = undef,
	Optional[Integer[0]]     $unit_job_timeout_sec          = undef,
	Optional[Enum[
			'none',
			'reboot',
			'reboot-force',
			'reboot-immediate',
			'poweroff',
			'poweroff-force',
			'poweroff-immediate'
		]] $unit_job_timeout_action = undef,
	Optional[String[1]]      $unit_job_timeout_reboot_arg   = undef,
	# unit conditions
	Optional[Pattern[
			/^\|?!?x86(-64)?$/,
			/^\|?!?ppc(-le)?$/,
			/^\|?!?ppc64(-le)?$/,
			/^\|?!?ia64$/,
			/^\|?!?parisc(64)?$/,
			/^\|?!?s390x?$/,
			/^\|?!?sparc(64)?$/,
			/^\|?!?mips(-le)?$/,
			/^\|?!?mips64(-le)?$/,
			/^\|?!?alpha$/,
			/^\|?!?arm(-be)?$/,
			/^\|?!?arm64(-be)?$/,
			/^\|?!?sh(64)?$/,
			/^\|?!?m86k$/,
			/^\|?!?tilegx$/,
			/^\|?!?cris$/
		]] $unit_condition_architecture = undef,
	Optional[Variant[
			Boolean,
			Pattern[/^\|?!?(vm|container)$/],
			Pattern[/^\|?!?(qemu|kvm|zvm|vmware|microsoft|oracle|xen|bochs|uml)$/],
			Pattern[/^\|?!?(openvz|lxc|lxc-libvirt|systemd-nspawn|docker)$/]
		]] $unit_condition_virtualization = undef,
	Optional[Pattern[/^\|?!?\S+$/]]
	                         $unit_condition_host           = undef,
	Optional[Pattern[/^\|?\S+$/]]
	                         $unit_condition_kernel_cmdline = undef,
	Optional[Pattern[/^\|?!?(selinux|apparmor|ima|smack|audit)$/]]
	                         $unit_condition_security       = undef,
	Optional[Pattern[
			/^\|?!?CAP_AUDIT_(CONTROL|READ|WRITE)$/,
			/^\|?!?CAP_BLOCK_SUSPEND$/,
			/^\|?!?CAP_CHOWN$/,
			/^\|?!?CAP_DAC_(OVERRIDE|READ_SEARCH)$/,
			/^\|?!?CAP_FOWNER$/,
			/^\|?!?CAP_FSETID$/,
			/^\|?!?CAP_IPC_(LOCK|OWNER)$/,
			/^\|?!?CAP_KILL$/,
			/^\|?!?CAP_LEASE$/,
			/^\|?!?CAP_LINUX_IMMUTABLE$/,
			/^\|?!?CAP_MAC_(ADMIN|OVERRIDE)$/,
			/^\|?!?CAP_MKNOD$/,
			/^\|?!?CAP_NET_(ADMIN|BIND_SERVICE|BROADCAST|RAW)$/,
			/^\|?!?CAP_SETGID$/,
			/^\|?!?CAP_SETFCAP$/,
			/^\|?!?CAP_SETUID$/,
			/^\|?!?CAP_SYS_(ADMIN|BOOT|CHROOT|MODULE|NICE|PACCT|PTRACE|RAWIO|RESOURCE|TIME|TTY_CONFIG)$/,
			/^\|?!?CAP_SYSLOG$/,
			/^\|?!?CAP_WAKE_ALARM$/
		]] $unit_condition_capability = undef,
	Optional[Boolean]        $unit_condition_ac_power       = undef,
	Optional[Pattern[/^\|?!?\/(etc|var)$/]]
	                         $unit_condition_needs_update   = undef,
	Optional[Boolean]        $unit_condition_first_boot     = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_condition_path_exists    = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_condition_path_exists_gl = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_condition_path_is_dir    = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_condition_path_is_syml   = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_condition_path_is_mntpt  = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_condition_path_is_rw     = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_condition_dir_not_empty  = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_condition_file_not_empty = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_condition_file_is_exec   = undef,
	# unit assertion
	Optional[Pattern[
			/^\|?!?x86(-64)?$/,
			/^\|?!?ppc(-le)?$/,
			/^\|?!?ppc64(-le)?$/,
			/^\|?!?ia64$/,
			/^\|?!?parisc(64)?$/,
			/^\|?!?s390x?$/,
			/^\|?!?sparc(64)?$/,
			/^\|?!?mips(-le)?$/,
			/^\|?!?mips64(-le)?$/,
			/^\|?!?alpha$/,
			/^\|?!?arm(-be)?$/,
			/^\|?!?arm64(-be)?$/,
			/^\|?!?sh(64)?$/,
			/^\|?!?m86k$/,
			/^\|?!?tilegx$/,
			/^\|?!?cris$/
		]] $unit_assert_architecture = undef,
	Optional[Variant[
			Boolean,
			Pattern[/^\|?!?(vm|container)$/],
			Pattern[/^\|?!?(qemu|kvm|zvm|vmware|microsoft|oracle|xen|bochs|uml)$/],
			Pattern[/^\|?!?(openvz|lxc|lxc-libvirt|systemd-nspawn|docker)$/]
		]] $unit_assert_virtualization = undef,
	Optional[Pattern[/^\|?!?\S+$/]]
	                         $unit_assert_host              = undef,
	Optional[Pattern[/^\|?\S+$/]]
	                         $unit_assert_kernel_cmdline    = undef,
	Optional[Pattern[/^\|?!?(selinux|apparmor|ima|smack|audit)$/]]
	                         $unit_assert_security          = undef,
	Optional[Pattern[
			/^\|?!?CAP_AUDIT_(CONTROL|READ|WRITE)$/,
			/^\|?!?CAP_BLOCK_SUSPEND$/,
			/^\|?!?CAP_CHOWN$/,
			/^\|?!?CAP_DAC_(OVERRIDE|READ_SEARCH)$/,
			/^\|?!?CAP_FOWNER$/,
			/^\|?!?CAP_FSETID$/,
			/^\|?!?CAP_IPC_(LOCK|OWNER)$/,
			/^\|?!?CAP_KILL$/,
			/^\|?!?CAP_LEASE$/,
			/^\|?!?CAP_LINUX_IMMUTABLE$/,
			/^\|?!?CAP_MAC_(ADMIN|OVERRIDE)$/,
			/^\|?!?CAP_MKNOD$/,
			/^\|?!?CAP_NET_(ADMIN|BIND_SERVICE|BROADCAST|RAW)$/,
			/^\|?!?CAP_SETGID$/,
			/^\|?!?CAP_SETFCAP$/,
			/^\|?!?CAP_SETUID$/,
			/^\|?!?CAP_SYS_(ADMIN|BOOT|CHROOT|MODULE|NICE|PACCT|PTRACE|RAWIO|RESOURCE|TIME|TTY_CONFIG)$/,
			/^\|?!?CAP_SYSLOG$/,
			/^\|?!?CAP_WAKE_ALARM$/
		]] $unit_assert_capability = undef,
	Optional[Boolean]        $unit_assert_ac_power          = undef,
	Optional[Pattern[/^\|?!?\/(etc|var)$/]]
	                         $unit_assert_needs_update      = undef,
	Optional[Boolean]        $unit_assert_first_boot        = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_assert_path_exists       = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_assert_path_exists_gl    = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_assert_path_is_dir       = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_assert_path_is_syml      = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_assert_path_is_mntpt     = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_assert_path_is_rw        = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_assert_dir_not_empty     = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_assert_file_not_empty    = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_assert_file_is_exec      = undef,
	Optional[String[1]]      $unit_source_path              = undef,
	# I N S T A L L
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $install_alias                 = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $install_wanted_by             = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $install_required_by           = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $install_also                  = undef,
	Optional[Pattern[/^\S+$/]]
	                         $install_default_instance      = undef,
	# custom sections
	Optional[Hash[String[1], Hash[String[1], Any, 1], 1]]
	                         $custom_options                = undef,
	# S E R V I C E
	# native [Service] section settings
	Optional[Enum['simple', 'forking', 'oneshot', 'dbus', 'notify', 'idle']]
	                         $service_type                  = undef,
	Optional[Boolean]        $service_remain_after_exit     = undef,
	Optional[Boolean]        $service_guess_main_pid        = undef,
	Optional[Pattern[/^\/\S+$/]]
	                         $service_pid_file              = undef,
	Optional[Pattern[/^\S+$/]]
	                         $service_bus_name              = undef,
	Optional[Array[Struct[{
			bus_name                      => Pattern[/^\S+$/],
			permission                    => Enum['see', 'talk', 'own'],
		}], 1]] $service_bus_policy = undef,
	Optional[Array[String, 1]]
	                         $service_exec_start_pre        = undef,
	Optional[Array[String, 1]]
	                         $service_exec_start            = undef,
	Optional[Array[String, 1]]
	                         $service_exec_start_post       = undef,
	Optional[Array[String, 1]]
	                         $service_exec_reload           = undef,
	Optional[Array[String, 1]]
	                         $service_exec_stop             = undef,
	Optional[Array[String, 1]]
	                         $service_exec_stop_post        = undef,
	Optional[Integer[0]]     $service_restart_sec           = undef,
	Optional[Integer[0]]     $service_timeout_start_sec     = undef,
	Optional[Integer[0]]     $service_timeout_stop_sec      = undef,
	Optional[Integer[0]]     $service_timeout_sec           = undef,
	Optional[Integer[0]]     $service_watchdog_sec          = undef,
	Optional[Enum['no', 'on-success', 'on-failure', 'on-abnormal', 'on-watchdog', 'on-abort', 'always']]
	                         $service_restart               = undef,
	Optional[Array[Variant[
			Integer[0, 255],
			Enum[
				'SIGHUP',
				'SIGINT',
				'SIGKILL',
				'SIGPIPE',
				'SIGALRM',
				'SIGTERM'
			]
		]]] $service_success_exit_status = undef,
	Optional[Array[Variant[
			Integer[0, 255],
			Enum[
				'SIGHUP',
				'SIGINT',
				'SIGKILL',
				'SIGPIPE',
				'SIGALRM',
				'SIGTERM'
			]
		]]] $service_restart_prevnt_exit_s = undef,
	Optional[Array[Variant[
			Integer[0, 255],
			Enum[
				'SIGHUP',
				'SIGINT',
				'SIGKILL',
				'SIGPIPE',
				'SIGALRM',
				'SIGTERM'
			]
		]]] $service_force_exit_status = undef,
	Optional[Boolean]        $service_root_dir_start_only   = undef,
	Optional[Boolean]        $service_non_blocking          = undef,
	Optional[Enum['none', 'main', 'all']]
	                         $service_notify_access         = undef,
	Optional[Array[Pattern[/^$/, /^\S+$/], 1]]
	                         $service_sockets               = undef,
	Optional[Integer[0]]     $service_start_limit_interval  = undef,
	Optional[Integer[0]]     $service_start_limit_burst     = undef,
	Optional[Enum[
			'none',
			'reboot',
			'reboot-force',
			'reboot-immediate',
			'poweroff',
			'poweroff-force',
			'poweroff-immediate'
		]] $service_start_limit_action = undef,
	Optional[Enum[
			'none',
			'reboot',
			'reboot-force',
			'reboot-immediate',
			'poweroff',
			'poweroff-force',
			'poweroff-immediate'
		]] $service_failure_action = undef,
	Optional[String]         $service_reboot_argument       = undef,
	Optional[Integer[0]]     $service_fd_store_max          = undef,
	# systemd.exec:
	Optional[Pattern[/^\/$/, /^\/\S+\/$/]]
	                         $service_working_directory     = undef,
	Optional[Pattern[/^\/$/, /^\/\S+\/$/]]
	                         $service_root_directory        = undef,
	Optional[Variant[Integer[0], Pattern[/^\S+$/]]]
	                         $service_user                  = undef,
	Optional[Variant[Integer[0], Pattern[/^\S+$/]]]
	                         $service_group                 = undef,
	Optional[Array[Variant[Integer[0], Pattern[/^$/, /^\S+$/]], 1]]
	                         $service_supplementary_groups  = undef,
	Optional[Integer[-20, 19]]
	                         $service_nice                  = undef,
	Optional[Integer[-1000, 1000]]
	                         $service_oom_score_adjust      = undef,
	Optional[Variant[Integer[0, 3], Enum['none', 'realtime', 'best-effort', 'idle']]]
	                         $service_io_scheduling_class   = undef,
	Optional[Integer[0, 7]]  $service_io_scheduling_prio    = undef,
	Optional[Enum['other', 'batch', 'idle', 'fifo', 'rr']]
	                         $service_cpu_sched_policy      = undef,
	Optional[Integer[0, 99]] $service_cpu_sched_priority    = undef,
	Optional[Boolean]        $service_cpu_sched_rset_on_frk = undef,
	Optional[Array[Variant[Integer[0], Enum['']], 1]]
	                         $service_cpu_affinity          = undef,
	Optional[Pattern[/^[0-7]{4}$/]]
	                         $service_umask                 = undef,
	Optional[Array[Pattern[
			/^$/,
			/^"\S+=.*"$/,
			/^\S+=\S+$/
		], 1]] $service_environment = undef,
	Optional[Array[Pattern[
			/^-?\/\S+$/,
			/^$/
		], 1]] $service_environment_file = undef,
	Optional[Enum['null', 'tty', 'tty-force', 'tty-fail', 'socket']]
	                         $service_standard_input        = undef,
	Optional[Enum[
			'inherit',
			'tty',
			'journal',
			'syslog',
			'kmsg',
			'journal+console',
			'syslog+console',
			'kmsg+console',
			'socket',
			'null'
		]] $service_standard_output = undef,
	Optional[Enum[
			'inherit',
			'tty',
			'journal',
			'syslog',
			'kmsg',
			'journal+console',
			'syslog+console',
			'kmsg+console',
			'socket',
			'null'
		]] $service_standard_error = undef,
	Optional[String]         $service_tty_path              = undef,
	Optional[Boolean]        $service_tty_reset             = undef,
	Optional[Boolean]        $service_ttyv_hangup           = undef,
	Optional[Boolean]        $service_ttyvt_disallocate     = undef,
	Optional[Pattern[/^\S+$/]]
	                         $service_syslog_identifier     = undef,
	Optional[Pattern[
			/^kern$/,
			/^user$/,
			/^mail$/,
			/^daemon$/,
			/^auth$/,
			/^syslog$/,
			/^lpr$/,
			/^news$/,
			/^uucp$/,
			/^cron$/,
			/^authpriv$/,
			/^ftp$/,
			/^local[0-7]$/
		]] $service_syslog_facility = undef,
	Optional[Enum[
			'emerg',
			'alert',
			'crit',
			'err',
			'warning',
			'notice',
			'info',
			'debug'
		]] $service_syslog_level = undef,
	Optional[Boolean]        $service_syslog_level_prefix   = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $service_timer_slack_nsec      = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $service_limit_cpu             = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $service_limit_fsize           = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $service_limit_data            = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $service_limit_stack           = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $service_limit_core            = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $service_limit_rss             = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $service_limit_nofile          = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $service_limit_as              = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $service_limit_nproc           = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $service_limit_memlock         = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $service_limit_locks           = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $service_limit_sigpending      = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $service_limit_msgqueue        = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $service_limit_nice            = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $service_limit_rtprio          = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $service_limit_rttime          = undef,
	Optional[Pattern[/^\S+$/]]
	                         $service_pam_name              = undef,
	Optional[Array[Struct[{
			invert_selection              => Boolean,
			capabilities                  => Array[Pattern[
					/^CAP_AUDIT_(CONTROL|READ|WRITE)$/,
					/^CAP_BLOCK_SUSPEND$/,
					/^CAP_CHOWN$/,
					/^CAP_DAC_(OVERRIDE|READ_SEARCH)$/,
					/^CAP_FOWNER$/,
					/^CAP_FSETID$/,
					/^CAP_IPC_(LOCK|OWNER)$/,
					/^CAP_KILL$/,
					/^CAP_LEASE$/,
					/^CAP_LINUX_IMMUTABLE$/,
					/^CAP_MAC_(ADMIN|OVERRIDE)$/,
					/^CAP_MKNOD$/,
					/^CAP_NET_(ADMIN|BIND_SERVICE|BROADCAST|RAW)$/,
					/^CAP_SETGID$/,
					/^CAP_SETFCAP$/,
					/^CAP_SETUID$/,
					/^CAP_SYS_(ADMIN|BOOT|CHROOT|MODULE|NICE|PACCT|PTRACE|RAWIO|RESOURCE|TIME|TTY_CONFIG)$/,
					/^CAP_SYSLOG$/,
					/^CAP_WAKE_ALARM$/
				], 0],
		}], 1]] $service_capability_boundn_set = undef,
	Optional[Array[Pattern[
			/^keep-caps(-locked)?$/,
			/^no-setuid-fixup(-locked)?$/,
			/^noroot(-locked)?$/,
			/^$/
		], 1]] $service_secure_bits = undef,
	Optional[Array[Struct[{
			capabilities                  => Variant[
					Array[Enum['all'], 1, 1],
					Array[Variant[
							Pattern[
								/^cap_audit_(control|read|write)$/,
								/^cap_block_suspend$/,
								/^cap_chown$/,
								/^cap_dac_(override|read_search)$/,
								/^cap_fowner$/,
								/^cap_fsetid$/,
								/^cap_ipc_(lock|owner)$/,
								/^cap_kill$/,
								/^cap_lease$/,
								/^cap_linux_immutable$/,
								/^cap_mac_(admin|override)$/,
								/^cap_mknod$/,
								/^cap_net_(admin|bind_service|broadcast|raw)$/,
								/^cap_setgid$/,
								/^cap_setfcap$/,
								/^cap_setuid$/,
								/^cap_sys_(admin|boot|chroot|module|nice|pacct|ptrace|rawio|resource|time|tty_config)$/,
								/^cap_syslog$/,
								/^cap_wake_alarm$/
							],
							Integer[0]
						], 1]
				],
			action_list                   => Array[Variant[
					Struct[{
						operator                      => Enum['='],
						flag                          => Optional[Pattern[
								/^ei?p?$/,
								/^e?ip?$/,
								/^e?i?p$/
							]],
					}],
					Struct[{
						operator                      => Enum['+', '-'],
						flag                          => Pattern[/^ei?p?$/, /^e?ip?$/, /^e?i?p$/],
					}]
				], 1]
		}], 1]] $service_capabilities = undef,
	Optional[Array[Pattern[/^$/, /^\/\S+\/$/], 1]]
	                         $service_read_write_dirs       = undef,
	Optional[Array[Pattern[/^$/, /^-?\/\S+\/$/], 1]]
	                         $service_read_only_dirs        = undef,
	Optional[Array[Pattern[/^$/, /^-?\/\S+\/$/], 1]]
	                         $service_inaccessible_dirs     = undef,
	Optional[Boolean]        $service_private_tmp           = undef,
	Optional[Boolean]        $service_private_devices       = undef,
	Optional[Boolean]        $service_private_network       = undef,
	Optional[Variant[Boolean, Enum['full']]]
	                         $service_protect_system        = undef,
	Optional[Variant[Boolean, Enum['read-only']]]
	                         $service_protect_home          = undef,
	Optional[Enum['shared', 'slave', 'private']]
	                         $service_mount_flags           = undef,
	Optional[Pattern[/^\S+$/]]
	                         $service_utmp_identifier       = undef,
	Optional[Enum['init', 'login', 'user']]
	                         $service_utmp_mode             = undef,
	Optional[Pattern[/^-?\S+$/]]
	                         $service_se_linux_context      = undef,
	Optional[Pattern[/^-?\S+$/]]
	                         $service_app_armor_profile     = undef,
	Optional[Pattern[/^-?\S+$/, /^$/]]
	                         $service_smack_process_label   = undef,
	Optional[Boolean]        $service_ignore_sigpipe        = undef,
	Optional[Boolean]        $service_no_new_privileges     = undef,
	Optional[Array[Struct[{
			invert_selection              => Boolean,
			system_calls                  => Variant[
					Array[Enum[''], 1, 1],
					Array[Pattern[/^\S+$/], 1]
				],
		}], 1]] $service_system_call_filter = undef,
	Optional[Pattern[/^$/, /^\S+$/]]
	                         $service_system_call_error_no  = undef,
	Optional[Array[Enum['x86', 'x86-64', 'x32', 'arm', 'native'], 1]]
	                         $service_system_call_archs     = undef,
	Optional[Struct[{
			invert_selection              => Boolean,
			address_families              => Variant[
					Array[Enum[
						'AF_APPLETALK',
						'AF_ASH',
						'AF_ATMPVC',
						'AF_ATMSVC',
						'AF_AX25',
						'AF_BLUETOOTH',
						'AF_BRIDGE',
						'AF_DECnet',
						'AF_ECONET',
						'AF_INET',
						'AF_INET6',
						'AF_IPX',
						'AF_IRDA',
						'AF_KEY',
						'AF_LLC',
						'AF_LOCAL',
						'AF_MAX',
						'AF_NETBEUI',
						'AF_NETLINK',
						'AF_NETROM',
						'AF_PACKET',
						'AF_PPPOX',
						'AF_ROSE',
						'AF_ROUTE',
						'AF_SECURITY',
						'AF_SNA',
						'AF_TIPC',
						'AF_UNSPEC',
						'AF_WANPIPE',
						'AF_X25',
						'AF_UNIX'
					], 1, 31],
					Array[Enum[''], 1, 1]
				],
		}]] $service_restrict_address_fam  = undef,
	Optional[Pattern[/^x86(-64)?$/]]
	                         $service_personality           = undef,
	Optional[Array[Pattern[/^[^[:blank:]\/]+$/], 1]]
	                         $service_runtime_dir           = undef,
	Optional[Pattern[/^[0-7]{4}$/]]
	                         $service_runtime_dir_mode      = undef,
	# systemd.kill:
	Optional[Enum['control-group', 'process', 'mixed', 'none']]
	                         $service_kill_mode             = undef,
	Optional[Enum[
			'SIGHUP',
			'SIGINT',
			'SIGPIPE',
			'SIGALRM',
			'SIGTERM',
			'SIGUSR1',
			'SIGUSR2',
			'SIGKILL'
		]] $service_kill_signal = undef,
	Optional[Boolean]        $service_send_sighup           = undef,
	Optional[Boolean]        $service_send_sigkill          = undef,
	# settings imported from systemd.resource-control
	Optional[Boolean]        $service_cpu_accounting        = undef,
	Optional[Integer]        $service_cpu_shares            = undef,
	Optional[Integer]        $service_startup_cpu_shares    = undef,
	Optional[Integer[0]]     $service_cpu_quota             = undef,
	Optional[Boolean]        $service_memory_accounting     = undef,
	Optional[Integer[0]]     $service_memory_limit          = undef,
	Optional[Boolean]        $service_blk_io_accounting     = undef,
	Optional[Integer[10, 1000]]
	                         $service_blk_io_weight         = undef,
	Optional[Integer[10, 1000]]
	                         $service_startup_blk_io_weight = undef,
	Optional[Array[Struct[{
			device                        => String[1],
			weight                        => Integer[10, 1000],
		}], 1]] $service_blk_io_device_weight = undef,
	Optional[Array[Struct[{
			device                        => String[1],
			bandwidth_bytes_per_second    => Integer[0],
		}], 1]] $service_blk_io_read_bw = undef,
	Optional[Array[Struct[{
			device                        => String[1],
			bandwidth_bytes_per_second    => Integer[0],
		}], 1]] $service_blk_io_write_bw = undef,
	Optional[Struct[{
			device                        => String[1],
			access                        => Pattern[/^rw?m?$/, /^r?wm?$/, /^r?w?m$/],
		}]] $service_device_allow = undef,
	Optional[Enum['auto', 'closed', 'strict']]
	                         $service_device_policy         = undef,
	Optional[String[1]]      $service_slice                 = undef,
	Optional[Boolean]        $service_delegate              = undef,
	# P U P P E T   E N H A N C E M E N T S
	Boolean                  $manage_service_resource       = false,
	Variant[Default, Enum['stopped', 'running']]
	                         $ensure_service                = default,
	# concerning 'mask': `systemctl mask <unit>` creates /dev/null link
	Optional[Variant[Boolean, Enum['manual']]]
	                         $enable_service                = undef,
	Optional[Boolean]        $hasrestart                    = undef,
	Optional[Boolean]        $hasstatus                     = undef,
	Boolean                  $explicit_provider             = false,
	Boolean                  $refresh_service_unit_update   = false,
) {
	#                                                                        #
	# C O M M O N   U N I T   S E C T I O N   I M P L E M E N T A T I O N    #
	#                                                                        #
	
	# load (system-dependent) default values from params.pp
	include systemd::params
	
	# we do not create files with spaces in their name
	if ($title =~ /\s/) and ($path == default) {
		fail('title may not contain spaces unless path specified')
	}
	
	if ($title =~ /\//) and ($path == default) {
		fail('title may not contain slashes unless path specified')
	}
	
	# A T T R I B U T E S   F O R   U N I T   F I L E
	$_ensure                       = ($ensure == default) ? {
		true                          => 'file',
		default                       => $ensure ? {
			'present'                     => 'file',
			default                       => $ensure,
		},
	}
	

	$_prefix_name                  = $title

	
	$_dirname                      = $systemd::params::systemd_local_units_dir
	
	$_basename                     = ($path == default) ? {
		true                          => "${_prefix_name}.service",
		default                       => inline_template("<%= File.basename(scope['path']) %>"),
	}
	
	$_path                         = ($path == default) ? {
		true                          => "${_dirname}${_basename}",
		default                       => $path,
	}
	

	$_title                        = "${_prefix_name}_service"

	
	$_mode                         = ($mode == default) ? {
		true                          => $systemd::params::systemd_unit_default_mode,
		default                       => $mode,
	}
	
	$_owner                        = ($owner == default) ? {
		true                          => $systemd::params::systemd_unit_default_owner,
		default                       => $owner,
	}
	
	$_group                        = ($group == default) ? {
		true                          => $systemd::params::systemd_unit_default_group,
		default                       => $group,
	}
	
	$_content_template             = ($content_template == default) ? {
		true                          => $systemd::params::systemd_service_template,
		default                       => $content_template,
	}
	
	$_validate_cmd                 = ($validate_cmd == default) ? {
		true                          => $systemd::params::systemd_unit_validate_cmd,
		default                       => $validate_cmd,
	}
	
	# A T T R I B U T E S   F O R   . W A N T S /  ,   . R E Q U I R E S / &  . D /
	$_ensure_dir                   = ($ensure == default) ? {
		true                          => 'directory',
		default                       => $ensure ? {
			'present'                     => 'directory',
			default                       => $ensure,
		},
	}
	
	# <unit_name>.wants/ directory
	$_wants_dir_mode               = ($wants_dir_mode == default) ? {
		true                          => $systemd::params::systemd_unit_wants_def_mode,
		default                       => $wants_dir_mode,
	}
	
	$_wants_dir_owner              = ($wants_dir_owner == default) ? {
		true                          => $systemd::params::systemd_unit_wants_def_owner,
		default                       => $wants_dir_owner,
	}
	
	$_wants_dir_group              = ($wants_dir_group == default) ? {
		true                          => $systemd::params::systemd_unit_wants_def_group,
		default                       => $wants_dir_group,
	}
	
	# <unit_name>.requires/ directory
	$_requires_dir_mode            = ($requires_dir_mode == default) ? {
		true                          => $systemd::params::systemd_unit_req_def_mode,
		default                       => $requires_dir_mode,
	}
	
	$_requires_dir_owner           = ($requires_dir_owner == default) ? {
		true                          => $systemd::params::systemd_unit_req_def_owner,
		default                       => $requires_dir_owner,
	}
	
	$_requires_dir_group           = ($requires_dir_group == default) ? {
		true                          => $systemd::params::systemd_unit_req_def_group,
		default                       => $requires_dir_group,
	}
	
	# <unit_name>.d/ directory
	$_conf_dir_mode                = ($conf_dir_mode == default) ? {
		true                          => $systemd::params::systemd_unit_conf_def_mode,
		default                       => $conf_dir_mode,
	}
	
	$_conf_dir_owner               = ($conf_dir_owner == default) ? {
		true                          => $systemd::params::systemd_unit_conf_def_owner,
		default                       => $conf_dir_owner,
	}
	
	$_conf_dir_group               = ($conf_dir_group == default) ? {
		true                          => $systemd::params::systemd_unit_conf_def_group,
		default                       => $conf_dir_group,
	}
	
	# R E S O U R C E   D E F A U L T S
	# their values are independent from anything
	File {
		validate_replacement          => $validate_replacement,
		show_diff                     => $show_diff,
		backup                        => $backup,
	}
	
	# R E S O U R C E   D E C L A R A T I O N S
	if ($_ensure != undef) {
		file { $_title:
			ensure                        => $_ensure,
			path                          => $_path,
			mode                          => $_mode,
			owner                         => $_owner,
			group                         => $_group,
			content                       => template($_content_template),
			validate_cmd                  => $_validate_cmd,
		}
		
		if $manage_wants_dir {
			$_wants_dir_resource_title     = "${_title}_wants"
			
			# something like $wants_symlinks each loop here? meh..
			# 
			# you have define symlinks on your own
			# ensure before => Systemd::Unit[$title] if $_ensure_dir = absent
			#    or require => Systemd::Unit[$title] if $_ensure_dir = present
			
			file { $_wants_dir_resource_title:
				ensure                        => $_ensure_dir,
				path                          => "${_path}.wants/",
				recurse                       => $wants_dir_complete_control,
				purge                         => $wants_dir_complete_control,
				# maybe some jerk created further subdirs in <unit>.wants/
				force                         => ($wants_dir_complete_control) or ($_ensure_dir == 'absent'),
			}
		}
		
		if $manage_requires_dir {
			$_requires_dir_resource_title  = "${_title}_requires"
			
			file { $_requires_dir_resource_title:
				ensure                        => $_ensure_dir,
				path                          => "${_path}.requires/",
				recurse                       => $requires_dir_complete_control,
				purge                         => $requires_dir_complete_control,
				force                         => ($requires_dir_complete_control) or ($_ensure_dir == 'absent'),
			}
		}
		
		if $manage_conf_dir {
			$_conf_dir_resource_title      = "${_title}_d"
			
			file { $_conf_dir_resource_title:
				ensure                        => $_ensure_dir,
				path                          => "${_path}.d/",
				recurse                       => $conf_dir_complete_control,
				purge                         => $conf_dir_complete_control,
				force                         => ($conf_dir_complete_control) or ($_ensure_dir == 'absent'),
			}
		}
	# end of ensure-something-block
	}

	
	#                                                                        #
	# S E R V I C E   S P E C I F I C   I M P L E M E N T A T I O N          #
	#                                                                        #
	
	# I N P U T
	if (($service_type == 'oneshot') and ($service_exec_start[1] != undef)) {
		warning('type is "oneshot" - there are more than one ExecStart')
	}
	
	# A T T R I B U T E S
	$_ensure_service               = ($ensure_service == default) ? {
		true                          => ($ensure == default) ? {
			true                          => 'running',
			default                       => $ensure ? {
				'present'                     => 'running',
				'absent'                      => 'stopped',
				default                       => undef,
			}
		},
		default                       => $ensure_service,
	}
	
	$_provider                     = $explicit_provider ? {
		true                          => 'systemd',
		default                       => undef,
	}
	
	$_service_name                 = ($path == default) ? {
		true                          => $_basename,
		default                       => undef,
	}
	
	if ($manage_service_resource) {
		if ($_service_name == undef) {
			# various systemctl commands (e.g. enable/disable/is-enabled)
			# only accept unit _names_ (not paths)
			fail('cannot manage services their unit file is at a custom path')
		}
		
		case $_ensure_service {
			'running': {
				if ($refresh_service_unit_update) {
					File[$_title] ~> Service[$_title]
				} else {
					File[$_title] -> Service[$_title]
				}
			}
			'stopped': {
				# Even though the *.service is cached by the system manager
				# we stop the service before we delete the service defining file,
				# just in case something goes wrong.
				Service[$_title] -> File[$_title]
			}
		}
		
		service { $_title:
			name                          => $_service_name,
			ensure                        => $_ensure_service,
			enable                        => $enable_service,
			hasrestart                    => $hasrestart,
			hasstatus                     => $hasstatus,
			provider                      => $_provider,
		}
		
		if (Class['systemd::daemon'] != undef) {
		}
	}
}
# EOF
