# name: systemd
# desc: definition manifest for systemd swap type
# auth: Kai Burghardt <wiz at KaiBurghardt.de>
#  vim: set tw=140 ts=4 noet bs=eol,start,indent fdm=marker fdl=0:

# systemd/manifests/swap.pp is generated

define systemd::swap (
	# P U P P E T   R E S O U R C E   M A N A G E M E N T
	Optional[Variant[Enum['absent', 'present'], Default]]
	                         $ensure                        = default,
	# <prefix_name>.swap file (and dependent)
	Variant[Pattern[/^\/\S*[^[:blank:]\/]$/], Default]
	                         $path                          = default,
	Optional[Variant[String[1], Default]]
	                         $mode                          = default,
	Optional[Variant[Integer[0], String[1], Default]]
	                         $owner                         = default,
	Optional[Variant[Integer[0], String[1], Default]]
	                         $group                         = default,
	Variant[String[1], Default]
	                         $content_template              = default,
	Optional[Variant[String[1], Default]]
	                         $validate_cmd                  = undef,
	Optional[String[1]]
	                         $validate_replacement          = undef,
	Optional[Enum[
			'md5',
			'md5lite',
			'sha256',
			'sha256lite',
			'mtime',
			'ctime',
			'none'
		]] $checksum = undef,
	Optional[Boolean]        $show_diff                     = undef,
	Optional[Variant[Boolean, String]]
	                         $backup                        = undef,
	String                   $blame                         = $systemd::params::systemd_unit_default_owner,
	# <unit_name>.wants/ directory
	Boolean                  $manage_wants_dir              = false,
	Optional[Variant[String[1], Default]]
	                         $wants_dir_mode                = default,
	Optional[Variant[Integer[0], String[1], Default]]
	                         $wants_dir_owner               = default,
	Optional[Variant[Integer[0], String[1], Default]]
	                         $wants_dir_group               = default,
	Boolean                  $wants_dir_complete_control    = false,
	# <unit_name>.requires/ directory
	Boolean                  $manage_requires_dir           = false,
	Optional[Variant[String[1], Default]]
	                         $requires_dir_mode             = default,
	Optional[Variant[Integer[0], String[1], Default]]
	                         $requires_dir_owner            = default,
	Optional[Variant[Integer[0], String[1], Default]]
	                         $requires_dir_group            = default,
	Boolean                  $requires_dir_complete_control = false,
	# <unit_name>.d/ directory
	Boolean                  $manage_conf_dir               = false,
	Optional[Variant[String[1], Default]]
	                         $conf_dir_mode                 = default,
	Optional[Variant[Integer[0], String[1], Default]]
	                         $conf_dir_owner                = default,
	Optional[Variant[Integer[0], String[1], Default]]
	                         $conf_dir_group                = default,
	Boolean                  $conf_dir_complete_control     = false,
	# U N I T
	# unit metadata
	Optional[String]         $unit_description              = undef,
	Optional[Array[Pattern[/^(https?:\/\/|file:|info:|man:)\S+$/, /^$/], 1]]
	                         $unit_documentation            = undef,
	# unit dependencies
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_requires                 = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_requires_overridable     = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_requisite                = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_requisite_overridable    = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_wants                    = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_binds_to                 = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_part_of                  = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_conflicts                = undef,
	# unit ordering
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_before                   = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_after                    = undef,
	# unit behavior
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_on_failure               = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_propagates_reload_to     = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_reload_propagated_from   = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_joins_namespace_of       = undef,
	# absolute path (starts with slash)
	Optional[Array[Pattern[/^\/\S+$/], 1]]
	                         $unit_requires_mount_for       = undef,
	Optional[Enum[
			'fail',
			'replace',
			'replace-irreversibly',
			'isolate',
			'flush',
			'ignore-dependencies',
			'ignore-requirements'
		]] $unit_on_failure_job_mode = undef,
	Optional[Boolean]        $unit_ignore_on_isolate        = undef,
	Optional[Boolean]        $unit_ignore_on_snapshot       = undef,
	Optional[Boolean]        $unit_stop_when_unneeded       = undef,
	Optional[Boolean]        $unit_refuse_manual_start      = undef,
	Optional[Boolean]        $unit_refuse_manual_stop       = undef,
	Optional[Boolean]        $unit_allow_isolate            = undef,
	Optional[Boolean]        $unit_default_dependencies     = undef,
	Optional[Integer[0]]     $unit_job_timeout_sec          = undef,
	Optional[Enum[
			'none',
			'reboot',
			'reboot-force',
			'reboot-immediate',
			'poweroff',
			'poweroff-force',
			'poweroff-immediate'
		]] $unit_job_timeout_action = undef,
	Optional[String[1]]      $unit_job_timeout_reboot_arg   = undef,
	# unit conditions
	Optional[Pattern[
			/^\|?!?x86(-64)?$/,
			/^\|?!?ppc(-le)?$/,
			/^\|?!?ppc64(-le)?$/,
			/^\|?!?ia64$/,
			/^\|?!?parisc(64)?$/,
			/^\|?!?s390x?$/,
			/^\|?!?sparc(64)?$/,
			/^\|?!?mips(-le)?$/,
			/^\|?!?mips64(-le)?$/,
			/^\|?!?alpha$/,
			/^\|?!?arm(-be)?$/,
			/^\|?!?arm64(-be)?$/,
			/^\|?!?sh(64)?$/,
			/^\|?!?m86k$/,
			/^\|?!?tilegx$/,
			/^\|?!?cris$/
		]] $unit_condition_architecture = undef,
	Optional[Variant[
			Boolean,
			Pattern[/^\|?!?(vm|container)$/],
			Pattern[/^\|?!?(qemu|kvm|zvm|vmware|microsoft|oracle|xen|bochs|uml)$/],
			Pattern[/^\|?!?(openvz|lxc|lxc-libvirt|systemd-nspawn|docker)$/]
		]] $unit_condition_virtualization = undef,
	Optional[Pattern[/^\|?!?\S+$/]]
	                         $unit_condition_host           = undef,
	Optional[Pattern[/^\|?\S+$/]]
	                         $unit_condition_kernel_cmdline = undef,
	Optional[Pattern[/^\|?!?(selinux|apparmor|ima|smack|audit)$/]]
	                         $unit_condition_security       = undef,
	Optional[Pattern[
			/^\|?!?CAP_AUDIT_(CONTROL|READ|WRITE)$/,
			/^\|?!?CAP_BLOCK_SUSPEND$/,
			/^\|?!?CAP_CHOWN$/,
			/^\|?!?CAP_DAC_(OVERRIDE|READ_SEARCH)$/,
			/^\|?!?CAP_FOWNER$/,
			/^\|?!?CAP_FSETID$/,
			/^\|?!?CAP_IPC_(LOCK|OWNER)$/,
			/^\|?!?CAP_KILL$/,
			/^\|?!?CAP_LEASE$/,
			/^\|?!?CAP_LINUX_IMMUTABLE$/,
			/^\|?!?CAP_MAC_(ADMIN|OVERRIDE)$/,
			/^\|?!?CAP_MKNOD$/,
			/^\|?!?CAP_NET_(ADMIN|BIND_SERVICE|BROADCAST|RAW)$/,
			/^\|?!?CAP_SETGID$/,
			/^\|?!?CAP_SETFCAP$/,
			/^\|?!?CAP_SETUID$/,
			/^\|?!?CAP_SYS_(ADMIN|BOOT|CHROOT|MODULE|NICE|PACCT|PTRACE|RAWIO|RESOURCE|TIME|TTY_CONFIG)$/,
			/^\|?!?CAP_SYSLOG$/,
			/^\|?!?CAP_WAKE_ALARM$/
		]] $unit_condition_capability = undef,
	Optional[Boolean]        $unit_condition_ac_power       = undef,
	Optional[Pattern[/^\|?!?\/(etc|var)$/]]
	                         $unit_condition_needs_update   = undef,
	Optional[Boolean]        $unit_condition_first_boot     = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_condition_path_exists    = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_condition_path_exists_gl = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_condition_path_is_dir    = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_condition_path_is_syml   = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_condition_path_is_mntpt  = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_condition_path_is_rw     = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_condition_dir_not_empty  = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_condition_file_not_empty = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_condition_file_is_exec   = undef,
	# unit assertion
	Optional[Pattern[
			/^\|?!?x86(-64)?$/,
			/^\|?!?ppc(-le)?$/,
			/^\|?!?ppc64(-le)?$/,
			/^\|?!?ia64$/,
			/^\|?!?parisc(64)?$/,
			/^\|?!?s390x?$/,
			/^\|?!?sparc(64)?$/,
			/^\|?!?mips(-le)?$/,
			/^\|?!?mips64(-le)?$/,
			/^\|?!?alpha$/,
			/^\|?!?arm(-be)?$/,
			/^\|?!?arm64(-be)?$/,
			/^\|?!?sh(64)?$/,
			/^\|?!?m86k$/,
			/^\|?!?tilegx$/,
			/^\|?!?cris$/
		]] $unit_assert_architecture = undef,
	Optional[Variant[
			Boolean,
			Pattern[/^\|?!?(vm|container)$/],
			Pattern[/^\|?!?(qemu|kvm|zvm|vmware|microsoft|oracle|xen|bochs|uml)$/],
			Pattern[/^\|?!?(openvz|lxc|lxc-libvirt|systemd-nspawn|docker)$/]
		]] $unit_assert_virtualization = undef,
	Optional[Pattern[/^\|?!?\S+$/]]
	                         $unit_assert_host              = undef,
	Optional[Pattern[/^\|?\S+$/]]
	                         $unit_assert_kernel_cmdline    = undef,
	Optional[Pattern[/^\|?!?(selinux|apparmor|ima|smack|audit)$/]]
	                         $unit_assert_security          = undef,
	Optional[Pattern[
			/^\|?!?CAP_AUDIT_(CONTROL|READ|WRITE)$/,
			/^\|?!?CAP_BLOCK_SUSPEND$/,
			/^\|?!?CAP_CHOWN$/,
			/^\|?!?CAP_DAC_(OVERRIDE|READ_SEARCH)$/,
			/^\|?!?CAP_FOWNER$/,
			/^\|?!?CAP_FSETID$/,
			/^\|?!?CAP_IPC_(LOCK|OWNER)$/,
			/^\|?!?CAP_KILL$/,
			/^\|?!?CAP_LEASE$/,
			/^\|?!?CAP_LINUX_IMMUTABLE$/,
			/^\|?!?CAP_MAC_(ADMIN|OVERRIDE)$/,
			/^\|?!?CAP_MKNOD$/,
			/^\|?!?CAP_NET_(ADMIN|BIND_SERVICE|BROADCAST|RAW)$/,
			/^\|?!?CAP_SETGID$/,
			/^\|?!?CAP_SETFCAP$/,
			/^\|?!?CAP_SETUID$/,
			/^\|?!?CAP_SYS_(ADMIN|BOOT|CHROOT|MODULE|NICE|PACCT|PTRACE|RAWIO|RESOURCE|TIME|TTY_CONFIG)$/,
			/^\|?!?CAP_SYSLOG$/,
			/^\|?!?CAP_WAKE_ALARM$/
		]] $unit_assert_capability = undef,
	Optional[Boolean]        $unit_assert_ac_power          = undef,
	Optional[Pattern[/^\|?!?\/(etc|var)$/]]
	                         $unit_assert_needs_update      = undef,
	Optional[Boolean]        $unit_assert_first_boot        = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_assert_path_exists       = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_assert_path_exists_gl    = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_assert_path_is_dir       = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_assert_path_is_syml      = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_assert_path_is_mntpt     = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_assert_path_is_rw        = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_assert_dir_not_empty     = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_assert_file_not_empty    = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_assert_file_is_exec      = undef,
	Optional[String[1]]      $unit_source_path              = undef,
	# I N S T A L L
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $install_alias                 = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $install_wanted_by             = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $install_required_by           = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $install_also                  = undef,
	Optional[Pattern[/^\S+$/]]
	                         $install_default_instance      = undef,
	# custom sections
	Optional[Hash[String[1], Hash[String[1], Any, 1], 1]]
	                         $custom_options                = undef,
	# S W A P
	# native [Swap] section settings
	Variant[Pattern[/^\/\S+$/], Default]
	                         $swap_what                     = default,
	Optional[Integer[-1, 32767]]
	                         $swap_priority                 = undef,
	Optional[Array[Pattern[/^[^[:blank:],]+$/], 1]]
	                         $swap_options                  = default,
	Optional[Integer[0]]     $swap_timeout_sec              = undef,
	# settings imported from systemd.exec
	Optional[Pattern[/^\/$/, /^\/\S+\/$/]]
	                         $swap_working_directory        = undef,
	Optional[Pattern[/^\/$/, /^\/\S+\/$/]]
	                         $swap_root_directory           = undef,
	Optional[Variant[Integer[0], Pattern[/^\S+$/]]]
	                         $swap_user                     = undef,
	Optional[Variant[Integer[0], Pattern[/^\S+$/]]]
	                         $swap_group                    = undef,
	Optional[Array[Variant[Integer[0], Pattern[/^$/, /^\S+$/]], 1]]
	                         $swap_supplementary_groups     = undef,
	Optional[Integer[-20, 19]]
	                         $swap_nice                     = undef,
	Optional[Integer[-1000, 1000]]
	                         $swap_oom_score_adjust         = undef,
	Optional[Variant[Integer[0, 3], Enum['none', 'realtime', 'best-effort', 'idle']]]
	                         $swap_io_scheduling_class      = undef,
	Optional[Integer[0, 7]]  $swap_io_scheduling_prio       = undef,
	Optional[Enum['other', 'batch', 'idle', 'fifo', 'rr']]
	                         $swap_cpu_sched_policy         = undef,
	Optional[Integer[0, 99]] $swap_cpu_sched_priority       = undef,
	Optional[Boolean]        $swap_cpu_sched_rset_on_frk    = undef,
	Optional[Array[Variant[Integer[0], Enum['']], 1]]
	                         $swap_cpu_affinity             = undef,
	Optional[Pattern[/^[0-7]{4}$/]]
	                         $swap_umask                    = undef,
	Optional[Array[Pattern[
			/^$/,
			/^"\S+=.*"$/,
			/^\S+=\S+$/
		], 1]] $swap_environment = undef,
	Optional[Array[Pattern[
			/^-?\/\S+$/,
			/^$/
		], 1]] $swap_environment_file = undef,
	Optional[Enum['null', 'tty', 'tty-force', 'tty-fail', 'socket']]
	                         $swap_standard_input           = undef,
	Optional[Enum[
			'inherit',
			'tty',
			'journal',
			'syslog',
			'kmsg',
			'journal+console',
			'syslog+console',
			'kmsg+console',
			'socket',
			'null'
		]] $swap_standard_output = undef,
	Optional[Enum[
			'inherit',
			'tty',
			'journal',
			'syslog',
			'kmsg',
			'journal+console',
			'syslog+console',
			'kmsg+console',
			'socket',
			'null'
		]] $swap_standard_error = undef,
	Optional[String]         $swap_tty_path                 = undef,
	Optional[Boolean]        $swap_tty_reset                = undef,
	Optional[Boolean]        $swap_ttyv_hangup              = undef,
	Optional[Boolean]        $swap_ttyvt_disallocate        = undef,
	Optional[Pattern[/^\S+$/]]
	                         $swap_syslog_identifier        = undef,
	Optional[Pattern[
			/^kern$/,
			/^user$/,
			/^mail$/,
			/^daemon$/,
			/^auth$/,
			/^syslog$/,
			/^lpr$/,
			/^news$/,
			/^uucp$/,
			/^cron$/,
			/^authpriv$/,
			/^ftp$/,
			/^local[0-7]$/
		]] $swap_syslog_facility = undef,
	Optional[Enum[
			'emerg',
			'alert',
			'crit',
			'err',
			'warning',
			'notice',
			'info',
			'debug'
		]] $swap_syslog_level = undef,
	Optional[Boolean]        $swap_syslog_level_prefix      = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $swap_timer_slack_nsec         = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $swap_limit_cpu                = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $swap_limit_fsize              = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $swap_limit_data               = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $swap_limit_stack              = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $swap_limit_core               = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $swap_limit_rss                = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $swap_limit_nofile             = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $swap_limit_as                 = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $swap_limit_nproc              = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $swap_limit_memlock            = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $swap_limit_locks              = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $swap_limit_sigpending         = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $swap_limit_msgqueue           = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $swap_limit_nice               = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $swap_limit_rtprio             = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $swap_limit_rttime             = undef,
	Optional[Pattern[/^\S+$/]]
	                         $swap_pam_name                 = undef,
	Optional[Array[Struct[{
			invert_selection              => Boolean,
			capabilities                  => Array[Pattern[
					/^CAP_AUDIT_(CONTROL|READ|WRITE)$/,
					/^CAP_BLOCK_SUSPEND$/,
					/^CAP_CHOWN$/,
					/^CAP_DAC_(OVERRIDE|READ_SEARCH)$/,
					/^CAP_FOWNER$/,
					/^CAP_FSETID$/,
					/^CAP_IPC_(LOCK|OWNER)$/,
					/^CAP_KILL$/,
					/^CAP_LEASE$/,
					/^CAP_LINUX_IMMUTABLE$/,
					/^CAP_MAC_(ADMIN|OVERRIDE)$/,
					/^CAP_MKNOD$/,
					/^CAP_NET_(ADMIN|BIND_SERVICE|BROADCAST|RAW)$/,
					/^CAP_SETGID$/,
					/^CAP_SETFCAP$/,
					/^CAP_SETUID$/,
					/^CAP_SYS_(ADMIN|BOOT|CHROOT|MODULE|NICE|PACCT|PTRACE|RAWIO|RESOURCE|TIME|TTY_CONFIG)$/,
					/^CAP_SYSLOG$/,
					/^CAP_WAKE_ALARM$/
				], 0],
		}], 1]] $swap_capability_boundn_set = undef,
	Optional[Array[Pattern[
			/^keep-caps(-locked)?$/,
			/^no-setuid-fixup(-locked)?$/,
			/^noroot(-locked)?$/,
			/^$/
		], 1]] $swap_secure_bits = undef,
	Optional[Array[Struct[{
			capabilities                  => Variant[
					Array[Enum['all'], 1, 1],
					Array[Variant[
							Pattern[
								/^cap_audit_(control|read|write)$/,
								/^cap_block_suspend$/,
								/^cap_chown$/,
								/^cap_dac_(override|read_search)$/,
								/^cap_fowner$/,
								/^cap_fsetid$/,
								/^cap_ipc_(lock|owner)$/,
								/^cap_kill$/,
								/^cap_lease$/,
								/^cap_linux_immutable$/,
								/^cap_mac_(admin|override)$/,
								/^cap_mknod$/,
								/^cap_net_(admin|bind_service|broadcast|raw)$/,
								/^cap_setgid$/,
								/^cap_setfcap$/,
								/^cap_setuid$/,
								/^cap_sys_(admin|boot|chroot|module|nice|pacct|ptrace|rawio|resource|time|tty_config)$/,
								/^cap_syslog$/,
								/^cap_wake_alarm$/
							],
							Integer[0]
						], 1]
				],
			action_list                   => Array[Variant[
					Struct[{
						operator                      => Enum['='],
						flag                          => Optional[Pattern[
								/^ei?p?$/,
								/^e?ip?$/,
								/^e?i?p$/
							]],
					}],
					Struct[{
						operator                      => Enum['+', '-'],
						flag                          => Pattern[/^ei?p?$/, /^e?ip?$/, /^e?i?p$/],
					}]
				], 1]
		}], 1]] $swap_capabilities = undef,
	Optional[Array[Pattern[/^$/, /^\/\S+\/$/], 1]]
	                         $swap_read_write_dirs          = undef,
	Optional[Array[Pattern[/^$/, /^-?\/\S+\/$/], 1]]
	                         $swap_read_only_dirs           = undef,
	Optional[Array[Pattern[/^$/, /^-?\/\S+\/$/], 1]]
	                         $swap_inaccessible_dirs        = undef,
	Optional[Boolean]        $swap_private_tmp              = undef,
	Optional[Boolean]        $swap_private_devices          = undef,
	Optional[Boolean]        $swap_private_network          = undef,
	Optional[Variant[Boolean, Enum['full']]]
	                         $swap_protect_system           = undef,
	Optional[Variant[Boolean, Enum['read-only']]]
	                         $swap_protect_home             = undef,
	Optional[Enum['shared', 'slave', 'private']]
	                         $swap_mount_flags              = undef,
	Optional[Pattern[/^\S+$/]]
	                         $swap_utmp_identifier          = undef,
	Optional[Enum['init', 'login', 'user']]
	                         $swap_utmp_mode                = undef,
	Optional[Pattern[/^-?\S+$/]]
	                         $swap_se_linux_context         = undef,
	Optional[Pattern[/^-?\S+$/]]
	                         $swap_app_armor_profile        = undef,
	Optional[Pattern[/^-?\S+$/, /^$/]]
	                         $swap_smack_process_label      = undef,
	Optional[Boolean]        $swap_ignore_sigpipe           = undef,
	Optional[Boolean]        $swap_no_new_privileges        = undef,
	Optional[Array[Struct[{
			invert_selection              => Boolean,
			system_calls                  => Variant[
					Array[Enum[''], 1, 1],
					Array[Pattern[/^\S+$/], 1]
				],
		}], 1]] $swap_system_call_filter = undef,
	Optional[Pattern[/^$/, /^\S+$/]]
	                         $swap_system_call_error_no     = undef,
	Optional[Array[Enum['x86', 'x86-64', 'x32', 'arm', 'native'], 1]]
	                         $swap_system_call_archs        = undef,
	Optional[Struct[{
			invert_selection              => Boolean,
			address_families              => Variant[
					Array[Enum[
						'AF_APPLETALK',
						'AF_ASH',
						'AF_ATMPVC',
						'AF_ATMSVC',
						'AF_AX25',
						'AF_BLUETOOTH',
						'AF_BRIDGE',
						'AF_DECnet',
						'AF_ECONET',
						'AF_INET',
						'AF_INET6',
						'AF_IPX',
						'AF_IRDA',
						'AF_KEY',
						'AF_LLC',
						'AF_LOCAL',
						'AF_MAX',
						'AF_NETBEUI',
						'AF_NETLINK',
						'AF_NETROM',
						'AF_PACKET',
						'AF_PPPOX',
						'AF_ROSE',
						'AF_ROUTE',
						'AF_SECURITY',
						'AF_SNA',
						'AF_TIPC',
						'AF_UNSPEC',
						'AF_WANPIPE',
						'AF_X25',
						'AF_UNIX'
					], 1, 31],
					Array[Enum[''], 1, 1]
				],
		}]] $swap_restrict_address_fam  = undef,
	Optional[Pattern[/^x86(-64)?$/]]
	                         $swap_personality              = undef,
	Optional[Array[Pattern[/^[^[:blank:]\/]+$/], 1]]
	                         $swap_runtime_dir              = undef,
	Optional[Pattern[/^[0-7]{4}$/]]
	                         $swap_runtime_dir_mode         = undef,
	# settings imported from systemd.kill
	Optional[Enum['control-group', 'process', 'mixed', 'none']]
	                         $swap_kill_mode                = undef,
	Optional[Enum[
			'SIGHUP',
			'SIGINT',
			'SIGPIPE',
			'SIGALRM',
			'SIGTERM',
			'SIGUSR1',
			'SIGUSR2',
			'SIGKILL'
		]] $swap_kill_signal = undef,
	Optional[Boolean]        $swap_send_sighup              = undef,
	Optional[Boolean]        $swap_send_sigkill             = undef,
	# settings imported from systemd.resource-control
	Optional[Boolean]        $swap_cpu_accounting           = undef,
	Optional[Integer]        $swap_cpu_shares               = undef,
	Optional[Integer]        $swap_startup_cpu_shares       = undef,
	Optional[Integer[0]]     $swap_cpu_quota                = undef,
	Optional[Boolean]        $swap_memory_accounting        = undef,
	Optional[Integer[0]]     $swap_memory_limit             = undef,
	Optional[Boolean]        $swap_blk_io_accounting        = undef,
	Optional[Integer[10, 1000]]
	                         $swap_blk_io_weight            = undef,
	Optional[Integer[10, 1000]]
	                         $swap_startup_blk_io_weight    = undef,
	Optional[Array[Struct[{
			device                        => String[1],
			weight                        => Integer[10, 1000],
		}], 1]] $swap_blk_io_device_weight = undef,
	Optional[Array[Struct[{
			device                        => String[1],
			bandwidth_bytes_per_second    => Integer[0],
		}], 1]] $swap_blk_io_read_bw = undef,
	Optional[Array[Struct[{
			device                        => String[1],
			bandwidth_bytes_per_second    => Integer[0],
		}], 1]] $swap_blk_io_write_bw = undef,
	Optional[Struct[{
			device                        => String[1],
			access                        => Pattern[/^rw?m?$/, /^r?wm?$/, /^r?w?m$/],
		}]] $swap_device_allow = undef,
	Optional[Enum['auto', 'closed', 'strict']]
	                         $swap_device_policy            = undef,
	Optional[String[1]]      $swap_slice                    = undef,
	Optional[Boolean]        $swap_delegate                 = undef,
	# P U P P E T   E N H A N C E M E N T S
	Boolean                  $manage_mount_resource         = false,
	Boolean                  $manage_service_resource       = false,
	Optional[Boolean]        $ensure_swapon                 = undef,
) {

	#                                                                        #
	# C O M M O N   U N I T   S E C T I O N   I M P L E M E N T A T I O N    #
	#                                                                        #
	
	# load (system-dependent) default values from params.pp
	include systemd::params
	
	# we do not create files with spaces in their name
	if ($title =~ /\s/) and ($path == default) {
		fail('title may not contain spaces unless path specified')
	}
	
	
	# A T T R I B U T E S   F O R   U N I T   F I L E
	$_ensure                       = ($ensure == default) ? {
		true                          => 'file',
		default                       => $ensure ? {
			'present'                     => 'file',
			default                       => $ensure,
		},
	}
	

	# if user explicitly specified "What=", then directly use $title
	# otherwise we assume $title is an absolute mount device path
	$_prefix_name                  = ($swap_what == default) ? {
		true                          => $title ? {
			'/'                           => '-',
			default                       =>
			inline_template("<%= @title.reverse.split('/').join('-').reverse.sub(/-$/, '') %>"),
		},
	    default                       => $title,
	}

	
	$_dirname                      = $systemd::params::systemd_local_units_dir
	
	$_basename                     = ($path == default) ? {
		true                          => "${_prefix_name}.swap",
		default                       => inline_template("<%= File.basename(scope['path']) %>"),
	}
	
	$_path                         = ($path == default) ? {
		true                          => "${_dirname}${_basename}",
		default                       => $path,
	}
	

	$_title                        = inline_template("<%= scope['_prefix_name'].gsub('-', '_') %>_swap")

	
	$_mode                         = ($mode == default) ? {
		true                          => $systemd::params::systemd_unit_default_mode,
		default                       => $mode,
	}
	
	$_owner                        = ($owner == default) ? {
		true                          => $systemd::params::systemd_unit_default_owner,
		default                       => $owner,
	}
	
	$_group                        = ($group == default) ? {
		true                          => $systemd::params::systemd_unit_default_group,
		default                       => $group,
	}
	
	$_content_template             = ($content_template == default) ? {
		true                          => $systemd::params::systemd_service_template,
		default                       => $content_template,
	}
	
	$_validate_cmd                 = ($validate_cmd == default) ? {
		true                          => $systemd::params::systemd_unit_validate_cmd,
		default                       => $validate_cmd,
	}
	
	# A T T R I B U T E S   F O R   . W A N T S /  ,   . R E Q U I R E S / &  . D /
	$_ensure_dir                   = ($ensure == default) ? {
		true                          => 'directory',
		default                       => $ensure ? {
			'present'                     => 'directory',
			default                       => $ensure,
		},
	}
	
	# <unit_name>.wants/ directory
	$_wants_dir_mode               = ($wants_dir_mode == default) ? {
		true                          => $systemd::params::systemd_unit_wants_def_mode,
		default                       => $wants_dir_mode,
	}
	
	$_wants_dir_owner              = ($wants_dir_owner == default) ? {
		true                          => $systemd::params::systemd_unit_wants_def_owner,
		default                       => $wants_dir_owner,
	}
	
	$_wants_dir_group              = ($wants_dir_group == default) ? {
		true                          => $systemd::params::systemd_unit_wants_def_group,
		default                       => $wants_dir_group,
	}
	
	# <unit_name>.requires/ directory
	$_requires_dir_mode            = ($requires_dir_mode == default) ? {
		true                          => $systemd::params::systemd_unit_req_def_mode,
		default                       => $requires_dir_mode,
	}
	
	$_requires_dir_owner           = ($requires_dir_owner == default) ? {
		true                          => $systemd::params::systemd_unit_req_def_owner,
		default                       => $requires_dir_owner,
	}
	
	$_requires_dir_group           = ($requires_dir_group == default) ? {
		true                          => $systemd::params::systemd_unit_req_def_group,
		default                       => $requires_dir_group,
	}
	
	# <unit_name>.d/ directory
	$_conf_dir_mode                = ($conf_dir_mode == default) ? {
		true                          => $systemd::params::systemd_unit_conf_def_mode,
		default                       => $conf_dir_mode,
	}
	
	$_conf_dir_owner               = ($conf_dir_owner == default) ? {
		true                          => $systemd::params::systemd_unit_conf_def_owner,
		default                       => $conf_dir_owner,
	}
	
	$_conf_dir_group               = ($conf_dir_group == default) ? {
		true                          => $systemd::params::systemd_unit_conf_def_group,
		default                       => $conf_dir_group,
	}
	
	# R E S O U R C E   D E F A U L T S
	# their values are independent from anything
	File {
		validate_replacement          => $validate_replacement,
		show_diff                     => $show_diff,
		backup                        => $backup,
	}
	
	# R E S O U R C E   D E C L A R A T I O N S
	if ($_ensure != undef) {
		file { $_title:
			ensure                        => $_ensure,
			path                          => $_path,
			mode                          => $_mode,
			owner                         => $_owner,
			group                         => $_group,
			content                       => template($_content_template),
			validate_cmd                  => $_validate_cmd,
		}
		
		if $manage_wants_dir {
			$_wants_dir_resource_title     = "${_title}_wants"
			
			# something like $wants_symlinks each loop here? meh..
			# 
			# you have define symlinks on your own
			# ensure before => Systemd::Unit[$title] if $_ensure_dir = absent
			#    or require => Systemd::Unit[$title] if $_ensure_dir = present
			
			file { $_wants_dir_resource_title:
				ensure                        => $_ensure_dir,
				path                          => "${_path}.wants/",
				recurse                       => $wants_dir_complete_control,
				purge                         => $wants_dir_complete_control,
				# maybe some jerk created further subdirs in <unit>.wants/
				force                         => ($wants_dir_complete_control) or ($_ensure_dir == 'absent'),
			}
		}
		
		if $manage_requires_dir {
			$_requires_dir_resource_title  = "${_title}_requires"
			
			file { $_requires_dir_resource_title:
				ensure                        => $_ensure_dir,
				path                          => "${_path}.requires/",
				recurse                       => $requires_dir_complete_control,
				purge                         => $requires_dir_complete_control,
				force                         => ($requires_dir_complete_control) or ($_ensure_dir == 'absent'),
			}
		}
		
		if $manage_conf_dir {
			$_conf_dir_resource_title      = "${_title}_d"
			
			file { $_conf_dir_resource_title:
				ensure                        => $_ensure_dir,
				path                          => "${_path}.d/",
				recurse                       => $conf_dir_complete_control,
				purge                         => $conf_dir_complete_control,
				force                         => ($conf_dir_complete_control) or ($_ensure_dir == 'absent'),
			}
		}
	# end of ensure-something-block
	}
	
	#                                                                        #
	# S W A P   S P E C I F I C   I M P L E M E N T A T I O N                #
	#                                                                        #
	
	$_service_ensure               = $ensure_swapon ? {
		true                          => 'running',
		false                         => 'stopped',
		default                       => undef,
	}
	
	if ($manage_mount_resource) {
		mount { $_title:
			ensure                         => absent,
			device                         => $swap_what,
			name                           => 'none',
			# avoid having declared the same swap at two places at any time
			before                         => File[$_title],
		}
	}
	
	if ($manage_service_resource) {
		service { $_title:
			name                           => $_basename,
			ensure                         => $_service_ensure,
			subscribe                      => File[$_title],
		}
	}
}
# EOF
