# name: systemd
# desc: definition manifest for systemd tmpfiles conf type
# auth: Kai Burghardt <wiz at KaiBurghardt.de>
#  vim: set tw=140 ts=4 noet bs=eol,start,indent fdm=marker fdl=0:

define systemd::tmpfiles_conf (
	# P U P P E T   R E S O U R C E   M A N A G E M E N T
	Optional[Variant[Enum['absent', 'present'], Default]]
	                         $ensure                        = default,
	# tmpfiles.conf
	Optional[Variant[Pattern[/^\/\S+[^[:blank:]\/]$/], Default]]
	                         $path                          = default,
	# tmpfile
	Optional[Array[Struct[{
			tmpfile_type                  => Pattern[
					/^[fF]!?$/,
					/^w!?$/,
					/^[dD]!?$/,
					/^p[+]?!?$/,
					/^L[+]?!?$/,
					/^c[+]?!?$/,
					/^b[+]?!?$/,
					/^C!?$/,
					/^[xX]!?$/,
					/^[rR]!?$/,
					/^[zZ]!?$/,
					/^[tT]!?$/,
					/^[hH]!?$/,
					/^a[+]!?$/,
					/^A[+]!?$/,
				],
			path                          => String[1],
			mode                          => Pattern[
					/^~?[0-7]{4}$/,
					/^-$/
				],
			uid                           => Variant[
					Integer[0],
					Pattern[
						/^\S+$/,
						/^-$/
					]
				],
			gid                           => Variant[
					Integer[0],
					Pattern[
						/^\S+$/,
						/^-$/
					]
				],
			age_in_seconds                => Variant[
					Integer[0],
					Enum['-']
				],
			arguments                     => String,
		}]]] $tmpfiles = undef,
) {
	#                                                                        #
	# I M P L E M E N T A T I O N                                            #
	#                                                                        #
	
	$_ensure                       = ($ensure == default) ? {
		true                          => 'file',
		default                       => $ensure ? {
			'present'                     => 'file',
			default                       => $ensure,
		},
	}
	
	# R E S O U R C E   D E C L A R A T I O N S
	if ($_ensure != undef) {
		file { $_title:
			ensure                        => $_ensure,
			path                          => $_path,
			mode                          => $_mode,
			owner                         => $_owner,
			group                         => $_group,
			content                       => template($_content_template),
		}
	}
}
# EOF
